/*
 * Created by Elizabeth Correa
 * Using the CC26X2R1 with the ADXL345 using I2C protocol
 * Created for Smart Health Monitoring 2020 - 2021 MQP Project/Team at Worcester Polytechnic Institute
 * Last updated: Feb. 13, 2021
 */

/*
 *  ======== i2ctmp.c ========
 */
#include <stdint.h>
#include <stddef.h>
#include <unistd.h>

/* Driver Header files */
#include <ti/drivers/GPIO.h>
#include <ti/drivers/I2C.h>
#include <ti/display/Display.h>

/* Driver configuration */
#include "ti_drivers_config.h"
#include "Board.h"

/* Temperature result registers */

#define TASKSTACKSIZE 640
#define X_Axis_Register_DATAX0 0x32 /*X-Axis Data 0 */
#define X_Axis_Register_DATAX1 0x33 /*X-Axis Data 1 */
#define Y_Axis_Register_DATAY0 0x34 /*Y-Axis Data 0 */
#define Y_Axis_Register_DATAY1 0x35 /*Y-Axis Data 1 */
#define Z_Axis_Register_DATAZ0 0x36 /*Z-Axis Data 0 */
#define Z_Axis_Register_DATAZ1 0x37 /*Z-Axis Data 1 */
#define DEVID 0x00 /* Device address */
#define PCR 0x2D /* power controlled register*/
static Display_Handle display;

#define TMP006_RESULT_REG          0x0001
#define TMP11X_RESULT_REG          0x0000

/* I2C slave addresses */
#define TMP006_LAUNCHPAD_ADDR      0x41
#define TMP11X_BASSENSORS_ADDR     0x48
#define TMP116_LAUNCHPAD_ADDR      0x49

/* Number of supported sensor iterations */
#define TMP_COUNT           3

/*
 * Data structure containing currently supported I2C TMP sensors.
 * Sensors are ordered by descending preference.
 */
static const struct {
    uint8_t address;
    uint8_t resultReg;
    char *id;

} sensors[TMP_COUNT] = {
    { TMP11X_BASSENSORS_ADDR, TMP11X_RESULT_REG, "11X" },
    { TMP116_LAUNCHPAD_ADDR,  TMP11X_RESULT_REG, "116" },
    { TMP006_LAUNCHPAD_ADDR,  TMP006_RESULT_REG, "006" }
};

static uint8_t slaveAddress;
static Display_Handle display;

static void i2cErrorHandler(I2C_Transaction *transaction,
    Display_Handle display);

/*
 *  ======== mainThread ========
 */
void *mainThread(void *arg0)
{

    int i; //Will be used for iterating through the different ADXL345 registers (in for loop)
    int x0,x1,y0,y1,z0,z1; //will be used for storing the data read from ADXL345 axes registers
    float x_out,y_out,z_out, newx_out, newy_out, newz_out; //will be used in the final reading computation for the axes
    int16_t xf,yf,zf; //will be used for combining the two byte readings for each axis into one 16 bit number
    uint8_t txBuffer[9];
    int interruptVariable; //will be used for determining if the different register bits have been triggered
    int freeFallValue, ActivityValue, inactivityValue;//will be used for determining which events have happened
    //i2c initialization variables
    I2C_Handle i2c;
    I2C_Params i2cParams;
    I2C_Transaction i2cTransaction;


    /* Call driver init functions */
    Display_init();
    GPIO_init();
    I2C_init();

    /* Configure the LED and if applicable, the TMP_EN pin */
    GPIO_setConfig(CONFIG_GPIO_LED_0, GPIO_CFG_OUT_STD | GPIO_CFG_OUT_LOW);
#ifdef CONFIG_GPIO_TMP_EN
    GPIO_setConfig(CONFIG_GPIO_TMP_EN, GPIO_CFG_OUT_STD | GPIO_CFG_OUT_HIGH);
    /* Allow the sensor to power on */
    sleep(1);
#endif

    /* Open the UART display for output */
    display = Display_open(Display_Type_UART, NULL);
    if (display == NULL) {
        while (1);
    }

    /* Turn on user LED */
    GPIO_write(CONFIG_GPIO_LED_0, CONFIG_GPIO_LED_ON);
    Display_printf(display, 0, 0, "Starting the i2ctmp example\n");

    /* Create I2C for usage */
    I2C_Params_init(&i2cParams);
    i2cParams.bitRate = I2C_400kHz; //This is a default value, this DOES NOT have to stay the same
    i2c = I2C_open(CONFIG_I2C_TMP, &i2cParams);
    if (i2c == NULL) {
        Display_printf(display, 0, 0, "Error Initializing I2C\n");
        while (1);
    }
    else {
        Display_printf(display, 0, 0, "I2C Initialized!\n");
    }
    /*


    ================= Initialization of fall detection Interrupt Registers goes here =================

    See bottom of Code for explanation and reference under the "Free-fall Evaluations" heading (second to last heading)

    Initializations of the following registers will go here (all commented for now while i test to ensure nothing breaks
        one at a time):

    0x24 -> 0x08 -> establishing it to have a 0.5g activity threshold (could also set it to 0x20 which is a 2g threshold)
    0x25 -> 0x03 -> set inactivity threshold to be 0.1875g
    0x26 -> 0x02 -> set inactivity time to be 2 seconds (could be also 0x0A for 10 seconds if we want more time)
    0x27 -> 0x7F/0xFF -> Enable Activity and Inactivity of X-, Y-, Z-axis, wherein Inactivity is ac-coupled mode,
                                Activity is dc-coupled/ac-coupled mode
    0x28 -> 0x0C -> Set free-fall threshold as 0.75g
    0x29 -> 0x06 -> Set free-fall threshold time as 30ms
    0x2C -> 0x0A -> Keep as default ODR = 100Hz
    0x2E -> 0x1C -> Enable Activity, Inactivity, Free-Fall interrupts
    0x31 -> 0x0B -> Set as ±16 g measurement range, 13-bit right alignment, high level interrupt trigger, I2C interface

    (Could also add some initializations for the axes offsets for calibration purposes here)

    This is all done in the same way we established the PCR (0x2D) and wrote to the register to put the ADXL in
        measurement mode (seen in code directly below, but pasted for convenience).
        All of these registers follow the same initialization I2C write transaction protocols:

        //populate the txBuffer before doing the I2C_transaction
        txBuffer[0] = [insert register address here];
        txBuffer[1] = [insert value being inserted into the register];


        Display_printf(display, 0, 0, "%x\n",[insert register address here]); //for debugging

        i2cTransaction.slaveAddress = (0x53); //address of the ADXL345 specified in data sheet
        //two bytes are going to be sent when doing a single byte write (Register + Value)
        i2cTransaction.writeBuf = txBuffer;
        i2cTransaction.writeCount = 2;
        //No reply from the slave when doing a write, no need for a buffer
        i2cTransaction.readBuf = NULL;
        i2cTransaction.readCount = 0;

        I2C_transfer(i2c, &i2cTransaction); //complete I2C transaction
*/

    /*
    ***** Implementation: ******

    ======= Activity Threshold Initialization =====
*/
        txBuffer[0] = 0x24; //Activity threshold
        txBuffer[1] = 0x03; //0.5g threshold

        i2cTransaction.slaveAddress = (0x53); //address of the ADXL345 specified in data sheet
        //two bytes are going to be sent when doing a single byte write (Register + Value)
        i2cTransaction.writeBuf = txBuffer;
        i2cTransaction.writeCount = 2;
        //No reply from the slave when doing a write, no need for a buffer
        i2cTransaction.readBuf = NULL;
        i2cTransaction.readCount = 0;

        I2C_transfer(i2c, &i2cTransaction);

//    ======= Inactivity Threshold Initialization =====

        txBuffer[0] = 0x25; //Inactivity threshold
        txBuffer[1] = 0x08; //0.1875g threshold

        i2cTransaction.slaveAddress = (0x53); //address of the ADXL345 specified in data sheet
        //two bytes are going to be sent when doing a single byte write (Register + Value)
        i2cTransaction.writeBuf = txBuffer;
        i2cTransaction.writeCount = 2;
        //No reply from the slave when doing a write, no need for a buffer
        i2cTransaction.readBuf = NULL;
        i2cTransaction.readCount = 0;

        I2C_transfer(i2c, &i2cTransaction);

//    ======= Inactivity Time Initialization =====

        txBuffer[0] = 0x26; //Activity threshold
        txBuffer[1] = 0x02; //2 second threshold

        i2cTransaction.slaveAddress = (0x53); //address of the ADXL345 specified in data sheet
        //two bytes are going to be sent when doing a single byte write (Register + Value)
        i2cTransaction.writeBuf = txBuffer;
        i2cTransaction.writeCount = 2;
        //No reply from the slave when doing a write, no need for a buffer
        i2cTransaction.readBuf = NULL;
        i2cTransaction.readCount = 0;

        I2C_transfer(i2c, &i2cTransaction);

//    ======= Enable Interrupts Initialization =====

        txBuffer[0] = 0x27; //Enable detection of activity and inactivity on all axes
        txBuffer[1] = 0xFF;

        i2cTransaction.slaveAddress = (0x53); //address of the ADXL345 specified in data sheet
        //two bytes are going to be sent when doing a single byte write (Register + Value)
        i2cTransaction.writeBuf = txBuffer;
        i2cTransaction.writeCount = 2;
        //No reply from the slave when doing a write, no need for a buffer
        i2cTransaction.readBuf = NULL;
        i2cTransaction.readCount = 0;

        I2C_transfer(i2c, &i2cTransaction);

//    ======= Free-Fall Threshold Initialization =====

        txBuffer[0] = 0x28; //Free-fall Threshold
        txBuffer[1] = 0x0C; //0.75g

        i2cTransaction.slaveAddress = (0x53); //address of the ADXL345 specified in data sheet
        //two bytes are going to be sent when doing a single byte write (Register + Value)
        i2cTransaction.writeBuf = txBuffer;
        i2cTransaction.writeCount = 2;
        //No reply from the slave when doing a write, no need for a buffer
        i2cTransaction.readBuf = NULL;
        i2cTransaction.readCount = 0;

        I2C_transfer(i2c, &i2cTransaction);

//    ======= Free-fall Time Initialization =====

        txBuffer[0] = 0x29; //Free-fall time threshold
        txBuffer[1] = 0x06; //30ms threshold

        i2cTransaction.slaveAddress = (0x53); //address of the ADXL345 specified in data sheet
        //two bytes are going to be sent when doing a single byte write (Register + Value)
        i2cTransaction.writeBuf = txBuffer;
        i2cTransaction.writeCount = 2;
        //No reply from the slave when doing a write, no need for a buffer
        i2cTransaction.readBuf = NULL;
        i2cTransaction.readCount = 0;

        I2C_transfer(i2c, &i2cTransaction);

//    ======= Output Data Rate Initialization =====

        txBuffer[0] = 0x2C; //Set ODR to be default 100Hz
        txBuffer[1] = 0x0A;

        i2cTransaction.slaveAddress = (0x53); //address of the ADXL345 specified in data sheet
        //two bytes are going to be sent when doing a single byte write (Register + Value)
        i2cTransaction.writeBuf = txBuffer;
        i2cTransaction.writeCount = 2;
        //No reply from the slave when doing a write, no need for a buffer
        i2cTransaction.readBuf = NULL;
        i2cTransaction.readCount = 0;

        I2C_transfer(i2c, &i2cTransaction);

 //   ======= Activity, Inactivity, and Free-Fall Interrupt Initialization =====

        txBuffer[0] = 0x2E; //Enable Activity, Inactivity, and free-fall interrupts
        txBuffer[1] = 0x1C;

        i2cTransaction.slaveAddress = (0x53); //address of the ADXL345 specified in data sheet
        //two bytes are going to be sent when doing a single byte write (Register + Value)
        i2cTransaction.writeBuf = txBuffer;
        i2cTransaction.writeCount = 2;
        //No reply from the slave when doing a write, no need for a buffer
        i2cTransaction.readBuf = NULL;
        i2cTransaction.readCount = 0;

        I2C_transfer(i2c, &i2cTransaction);

//    ======= Range/Alignment Initialization =====

        txBuffer[0] = 0x31; //Set as ±16 g measurement range, 13-bit right alignment,
                                //high level interrupt trigger, I2C interface
        txBuffer[1] = 0x0B;

        i2cTransaction.slaveAddress = (0x53); //address of the ADXL345 specified in data sheet
        //two bytes are going to be sent when doing a single byte write (Register + Value)
        i2cTransaction.writeBuf = txBuffer;
        i2cTransaction.writeCount = 2;
        //No reply from the slave when doing a write, no need for a buffer
        i2cTransaction.readBuf = NULL;
        i2cTransaction.readCount = 0;

        I2C_transfer(i2c, &i2cTransaction);



    //populate the txBuffer before doing the I2C_transaction
    txBuffer[0] = PCR; //0x2D Power_CTL (Read Write)
    //set bit 4 and 3 [0000 1000] for measurement mode enable (allows for reading of axes)
    txBuffer[1] = 0x08;


    Display_printf(display, 0, 0, "%x\n",PCR); //This only serves for debugging - can be deleted in final iteration

    i2cTransaction.slaveAddress = (0x53); //address of the ADXL345 specified in data sheet
    //two bytes are going to be sent when doing a single byte write (Register + Value)
    i2cTransaction.writeBuf = txBuffer;
    i2cTransaction.writeCount = 2;
    //No reply from the slave when doing a write, no need for a buffer
    i2cTransaction.readBuf = NULL;
    i2cTransaction.readCount = 0;

    //We do not need a reading transaction, since we are only writing to the register

    /*
     * If the i2c transaction for setting the ADXL345 in measurement mode (seen above) goes through without errors,
     * we can begin actually reading from the sensor.
     * If not, there is no point in trying to reading from the sensor
     */
    if (I2C_transfer(i2c, &i2cTransaction))
    {
    //Lets wait a second to let the device reset and start calculations after writing to the power control register
    sleep(1);

int j;
j = 0;

/*
 * The while loop only allows for the readings to be sent to serial 5 times (looping 5 times as when i was testing
 * i didnt want to be sent a million different values, i wanted to track them).
 * This can be changed to allow for different amounts of iterations or a while (1)
 * function or something like it after sensor testing,
 * to allow the accelerometer to loop and continue reading continuously
 */
while(j < 5){

/*
 * for this for loop, i have i start at 2 and end at 8 to signify which register we are reading from,
 * for example, i == 2 means that we are reading from register 0x32, which is the first 8 bits of the x axis reading
 * i == 3 means we are reading from 0x33, i == 4 means we are reading from 0x34, and so on
 */
    for(i=2;i<8;i++){

/*
 * ===================== I2C Transaction Code Description for Reading from the ADXL345 Axes Registers =======================*
 *
 * For all of these transactions of reading from these registers, it will take on the following protocol:
 * Even though we are not writing to these registers, we need to fulfill an i2c transaction where we
 * "write" nothing to the register we want in order to allow us to read from it. This is done by doing the following:
 *
 *      uint8_t localWritetxBuffer[1]; //we are only giving the write the address of the register thus it only needs one space in the array
        localWritetxBuffer[0] = [insert Axis Register Here]; //populate that first space with the register we want
        uint8_t localWriterxBuffer[1];
        i2cTransaction.slaveAddress = 0x53; //this is the address of the ADXL345 in the way we have it wired/set up

        i2cTransaction.writeBuf = localWritetxBuffer;
        i2cTransaction.writeCount = 1; //1 byte is going to be sent when doing a single byte read (Register)

        //We cannot read from the register yet. A full write transaction must be performed before any data can be read from it
        i2cTransaction.readBuf = localWriterxBuffer;
        i2cTransaction.readCount = 0;

        //call the end of the write transaction (A full write transaction must be performed before any data can be read from it)
        I2C_transfer(i2c, &i2cTransaction);

 *  Next, we can finally read data from the register, in the following code, we do not need to write anything to the register,
 *  but can now read from it:
 *
 *      i2cTransaction.slaveAddress = 0x53; //this is the address of the ADXL345 in the way we have it wired/set up

        //As stated earlier, we do not need to fulfill another write transaction, we can just read from it
        //Thus the i2cTransaction.writeCount can be set to 0
        i2cTransaction.writeBuf = localWritetxBuffer;
        i2cTransaction.writeCount = 0;

        //Now we can read from the register, and since we are reading in one byte
        //we can set i2cTransaction.readCount equal to 1.
        i2cTransaction.readBuf = localWriterxBuffer;
        i2cTransaction.readCount = 1;

        //If this transaction goes through without errors, we will then be able to store that byte into an integer
        //that will be used in the joining/masking process later (for combining the 2 bytes into the 2's complement 16 bit reading)

        if(I2C_transfer(i2c, &i2cTransaction)){;

        [insert integer variable here] = localWriterxBuffer[0]; //store the data reading into a variable

 */

//      =================== Code Implementation of Reading Data from Axes Registers ==================

    if(i == 2){ //Reading from register 0x32 -> x0 register (1st 8 bits from the x-axis reading)
        uint8_t localWritetxBuffer[1];
        localWritetxBuffer[0] = X_Axis_Register_DATAX0; //0x32
        uint8_t localWriterxBuffer[1];
        i2cTransaction.slaveAddress = 0x53;

        i2cTransaction.writeBuf = localWritetxBuffer;
        i2cTransaction.writeCount = 1;

        i2cTransaction.readBuf = localWriterxBuffer;
        i2cTransaction.readCount = 0;

        I2C_transfer(i2c, &i2cTransaction);

        //This sleep(1) is only here for debugging purposes, so that the serial is not slammed all at once with data
        //can be removed in final iterations
        sleep(1);

        i2cTransaction.slaveAddress = 0x53;

        i2cTransaction.writeBuf = localWritetxBuffer;
        i2cTransaction.writeCount = 0;

        i2cTransaction.readBuf = localWriterxBuffer;
        i2cTransaction.readCount = 1;

        //If this transaction fails, we cannot read from the register
        //This would mean that there was a bus fault, and would give us the error statement seen in the else statement below
        if(I2C_transfer(i2c, &i2cTransaction)){

        x0 = localWriterxBuffer[0];

        //Print the value once it is received, this is for debugging purposes to ensure we are getting values from the register

        Display_printf(display, 0, 0, "[%x]: %x \n", X_Axis_Register_DATAX0, x0);
        }
        else
        {
        Display_printf(display, 0, 0, "I2C Bus fault\n");
        }
        }

    if(i == 3){ //Reading from register 0x33 -> x1 register (Last 8 bits from the x-axis reading)
        uint8_t localWritetxBuffer[1];
        localWritetxBuffer[0] = X_Axis_Register_DATAX1; //0x33
        uint8_t localWriterxBuffer[1];
        i2cTransaction.slaveAddress = 0x53;

        i2cTransaction.writeBuf = localWritetxBuffer;
        i2cTransaction.writeCount = 1;

        i2cTransaction.readBuf = localWriterxBuffer;
        i2cTransaction.readCount = 0;

        I2C_transfer(i2c, &i2cTransaction);

        //This sleep(1) is only here for debugging purposes, so that the serial is not slammed all at once with data
        //can be removed in final iterations
        sleep(1);

        i2cTransaction.slaveAddress = 0x53;

        i2cTransaction.writeBuf = localWritetxBuffer;
        i2cTransaction.writeCount = 0;

        i2cTransaction.readBuf = localWriterxBuffer;
        i2cTransaction.readCount = 1;

        //If this transaction fails, we cannot read from the register
        //This would mean that there was a bus fault, and would give us the error statement seen in the else statement below
        if(I2C_transfer(i2c, &i2cTransaction)){

        //We want to shift this value 8 bits to the left, as it will be the most significant byte in the 16 bit data received
        x1 = (localWriterxBuffer[0] << 8);

        //Print the value once it is received, this is for debugging purposes to ensure we are getting values from the register

        Display_printf(display, 0, 0, "[%x]: %x \n", X_Axis_Register_DATAX1, x1);
        }
        else
        {
        Display_printf(display, 0, 0, "I2C Bus fault\n");
        }
        }

    if(i == 4){ //Reading from register 0x34 -> y0 register (1st 8 bits from the y-axis reading)
        uint8_t localWritetxBuffer[1];
        localWritetxBuffer[0] = Y_Axis_Register_DATAY0; //0x34
        uint8_t localWriterxBuffer[1];
        i2cTransaction.slaveAddress = 0x53;

        i2cTransaction.writeBuf = localWritetxBuffer;
        i2cTransaction.writeCount = 1;

        i2cTransaction.readBuf = localWriterxBuffer;
        i2cTransaction.readCount = 0;

        I2C_transfer(i2c, &i2cTransaction);

        //This sleep(1) is only here for debugging purposes, so that the serial is not slammed all at once with data
        //can be removed in final iterations
        sleep(1);

        i2cTransaction.slaveAddress = 0x53;

        i2cTransaction.writeBuf = localWritetxBuffer;
        i2cTransaction.writeCount = 0;

        i2cTransaction.readBuf = localWriterxBuffer;
        i2cTransaction.readCount = 1;

        //If this transaction fails, we cannot read from the register
        //This would mean that there was a bus fault, and would give us the error statement seen in the else statement below
        if(I2C_transfer(i2c, &i2cTransaction)){

        y0 = localWriterxBuffer[0];

        //Print the value once it is received, this is for debugging purposes to ensure we are getting values from the register

        Display_printf(display, 0, 0, "[%x]: %x \n", Y_Axis_Register_DATAY0, y0);
        }
        else
        {
        Display_printf(display, 0, 0, "I2C Bus fault\n");
        }
    }

    if(i == 5){  //Reading from register 0x35 -> y1 register (Last 8 bits from the y-axis reading)
        uint8_t localWritetxBuffer[1];
        localWritetxBuffer[0] = Y_Axis_Register_DATAY1; //0x35
        uint8_t localWriterxBuffer[1];
        i2cTransaction.slaveAddress = 0x53;
                        //1 byte is going to be sent when doing a single byte read (Register)
        i2cTransaction.writeBuf = localWritetxBuffer;
        i2cTransaction.writeCount = 1;
                        //1 byte is going to be sent when doing a single byte read (Register)
        i2cTransaction.readBuf = localWriterxBuffer;
        i2cTransaction.readCount = 0;

        I2C_transfer(i2c, &i2cTransaction);

        //This sleep(1) is only here for debugging purposes, so that the serial is not slammed all at once with data
        //can be removed in final iterations
        sleep(1);

        i2cTransaction.slaveAddress = 0x53;
                                //1 byte is going to be sent when doing a single byte read (Register)
        i2cTransaction.writeBuf = localWritetxBuffer;
        i2cTransaction.writeCount = 0;
                                //1 byte is going to be sent when doing a single byte read (Register)
        i2cTransaction.readBuf = localWriterxBuffer;
        i2cTransaction.readCount = 1;

        //If this transaction fails, we cannot read from the register
        //This would mean that there was a bus fault, and would give us the error statement seen in the else statement below
        if(I2C_transfer(i2c, &i2cTransaction)){

        //We want to shift this value 8 bits to the left, as it will be the most significant byte in the 16 bit data received
        y1 = (localWriterxBuffer[0] << 8);

        //Print the value once it is received, this is for debugging purposes to ensure we are getting values from the register
        Display_printf(display, 0, 0, "[%x]: %x \n", Y_Axis_Register_DATAY1, y1);
        }
        else
        {
        Display_printf(display, 0, 0, "I2C Bus fault\n");
        }
    }

    if(i == 6){ //Reading from register 0x36 -> z0 register (1st 8 bits from the z-axis reading)
        uint8_t localWritetxBuffer[1];
        localWritetxBuffer[0] = Z_Axis_Register_DATAZ0; //0x36
        uint8_t localWriterxBuffer[1];
        i2cTransaction.slaveAddress = 0x53;

        i2cTransaction.writeBuf = localWritetxBuffer;
        i2cTransaction.writeCount = 1;

        i2cTransaction.readBuf = localWriterxBuffer;
        i2cTransaction.readCount = 0;

        I2C_transfer(i2c, &i2cTransaction);

        //This sleep(1) is only here for debugging purposes, so that the serial is not slammed all at once with data
        //can be removed in final iterations
        sleep(1);

        i2cTransaction.slaveAddress = 0x53;

        i2cTransaction.writeBuf = localWritetxBuffer;
        i2cTransaction.writeCount = 0;

        i2cTransaction.readBuf = localWriterxBuffer;
        i2cTransaction.readCount = 1;

        //If this transaction fails, we cannot read from the register
        //This would mean that there was a bus fault, and would give us the error statement seen in the else statement below
        if(I2C_transfer(i2c, &i2cTransaction)){


        z0 = localWriterxBuffer[0];

        //Print the value once it is received, this is for debugging purposes to ensure we are getting values from the register
        Display_printf(display, 0, 0, "[%x]: %x \n", Z_Axis_Register_DATAZ0, z0);
        }
        else
        {
        Display_printf(display, 0, 0, "I2C Bus fault\n");
        }
    }

    if(i == 7){ //Reading from register 0x37 -> z1 register (Last 8 bits from the z-axis reading)
        uint8_t localWritetxBuffer[1];
        localWritetxBuffer[0] = Z_Axis_Register_DATAZ1; //0x37
        uint8_t localWriterxBuffer[1];
        i2cTransaction.slaveAddress = 0x53;

        i2cTransaction.writeBuf = localWritetxBuffer;
        i2cTransaction.writeCount = 1;

        i2cTransaction.readBuf = localWriterxBuffer;
        i2cTransaction.readCount = 0;

        I2C_transfer(i2c, &i2cTransaction);

        //This sleep(1) is only here for debugging purposes, so that the serial is not slammed all at once with data
        //can be removed in final iterations
        sleep(1);

        i2cTransaction.slaveAddress = 0x53;

        i2cTransaction.writeBuf = localWritetxBuffer;
        i2cTransaction.writeCount = 0;

        i2cTransaction.readBuf = localWriterxBuffer;
        i2cTransaction.readCount = 1;

        //If this transaction fails, we cannot read from the register
        //This would mean that there was a bus fault, and would give us the error statement seen in the else statement below
        if(I2C_transfer(i2c, &i2cTransaction)){

        //We want to shift this value 8 bits to the left, as it will be the most significant byte in the 16 bit data received
        z1 = (localWriterxBuffer[0] << 8);

        //Print the value once it is received, this is for debugging purposes to ensure we are getting values from the register
        Display_printf(display, 0, 0, "[%x]: %x \n", Z_Axis_Register_DATAZ1, z1);
        }
        else
        {
        Display_printf(display, 0, 0, "I2C Bus fault\n");
        }
    }

//This sleep(1) is only here for debugging purposes, so that the serial is not slammed all at once with data
//can be removed in final iterations
sleep(1);

} //end of for loop

//Since the x1 was masked to the left 8 bits already, we can just add the two bytes together to get the 16 bit data we need
//Then multiply it by the 4mg/LSB as specified by the ADXL345 data sheet, which is determined by the range we used
xf=(x0)+(x1);
x_out = xf*0.004;


//Since the y1 was masked to the left 8 bits already, we can just add the two bytes together to get the 16 bit data we need
//Then multiply it by the 4mg/LSB as specified by the ADXL345 data sheet, which is determined by the range we used
yf=(y0)+(y1);
y_out = yf*0.004;


//Since the z1 was masked to the left 8 bits already, we can just add the two bytes together to get the 16 bit data we need
//Then multiply it by the 4mg/LSB as specified by the ADXL345 data sheet, which is determined by the range we used
zf=(z0)+(z1);
z_out = zf*0.004;

//Now print these values to the serial. Used for debugging purposes, can be removed in final iterations
Display_printf(display, 0, 0, "x [%d]: %f g\n", xf, x_out);
Display_printf(display, 0, 0, "y [%d]: %f g\n", yf, y_out);
Display_printf(display, 0, 0, "z [%d]: %f g\n", zf, z_out);

/*
 * =========================== Determining if a Fall has Happened =======================
 *
 * Insert Free-fall detection determinations here (see below) titled "Free-fall Evaluations" for a more in-depth
 *      understanding of the free-fall evaluation process
 *
 * Here we will check the various interrupt pins to see if they were activated
 *
 * The first step is checking the register 0x30 to see if the free-fall interrupt has been triggered
 * (perform a read transaction like the one we have seen with the axes registers)
 */

        uint8_t localWritetxBuffer[1];
        localWritetxBuffer[0] = 0x30;
        uint8_t localWriterxBuffer[1];
        i2cTransaction.slaveAddress = 0x53;
        i2cTransaction.writeBuf = localWritetxBuffer;
        i2cTransaction.writeCount = 1;
        i2cTransaction.readBuf = localWriterxBuffer;
        i2cTransaction.readCount = 0;

        I2C_transfer(i2c, &i2cTransaction);

        //This sleep(1) is only here for debugging purposes, so that the serial is not slammed all at once with data
        //can be removed in final iterations
        sleep(1);

        i2cTransaction.slaveAddress = 0x53;
        i2cTransaction.writeBuf = localWritetxBuffer;
        i2cTransaction.writeCount = 0;
        i2cTransaction.readBuf = localWriterxBuffer;
        i2cTransaction.readCount = 1;

        //If this transaction fails, we cannot read from the register
        //This would mean that there was a bus fault, and would give us the error statement seen in the else statement below
        if(I2C_transfer(i2c, &i2cTransaction)){


        interruptVariable = localWriterxBuffer[0];

        Display_printf(display, 0, 0, "[%x]: %x \n", 0x30, interruptVariable);
        }
        else
        {
        Display_printf(display, 0, 0, "I2C Bus fault\n");
        }

        freeFallValue = (interruptVariable & 0b00000100);
        if(freeFallValue == 4){
            Display_printf(display, 0, 0, "Free-fall event activated\n");
            sleep(0.5);

 /*
 *  perform another i2C transaction checking 0x30, this time for an activity interrupt
 *  (seen above with for the before the freefall if statement)
 */

            uint8_t localWritetxBuffer[0] = 0x30;
            uint8_t localWriterxBuffer[1];
            i2cTransaction.slaveAddress = 0x53;
            i2cTransaction.writeBuf = localWritetxBuffer;
            i2cTransaction.writeCount = 1;
            i2cTransaction.readBuf = localWriterxBuffer;
            i2cTransaction.readCount = 0;

            I2C_transfer(i2c, &i2cTransaction);

        //This sleep(1) is only here for debugging purposes, so that the serial is not slammed all at once with data
        //can be removed in final iterations
            sleep(1);

            i2cTransaction.slaveAddress = 0x53;
            i2cTransaction.writeBuf = localWritetxBuffer;
            i2cTransaction.writeCount = 0;
            i2cTransaction.readBuf = localWriterxBuffer;
            i2cTransaction.readCount = 1;

        //If this transaction fails, we cannot read from the register
        //This would mean that there was a bus fault, and would give us the error statement seen in the else statement below
            if(I2C_transfer(i2c, &i2cTransaction)){


                interruptVariable = localWriterxBuffer[0];

                Display_printf(display, 0, 0, "[%x]: %x \n", 0x30, interruptVariable);
            }
            else
            {
                Display_printf(display, 0, 0, "I2C Bus fault\n");
            }

            ActivityValue == (interruptVariable & 0b00010000);
            if(ActivityValue == 16){
                Display_printf(display, 0, 0, "Activity sensed\n");
                sleep(0.5);

 /*
 *  perform another i2c transaction checking 0x30, this time for inactivity interrupt
 *  (seen above with for the before the freefall if statement)
 */

                uint8_t localWritetxBuffer[0] = 0x30;
                uint8_t localWriterxBuffer[1];
                i2cTransaction.slaveAddress = 0x53;
                i2cTransaction.writeBuf = localWritetxBuffer;
                i2cTransaction.writeCount = 1;
                i2cTransaction.readBuf = localWriterxBuffer;
                i2cTransaction.readCount = 0;

                I2C_transfer(i2c, &i2cTransaction);

        //This sleep(1) is only here for debugging purposes, so that the serial is not slammed all at once with data
        //can be removed in final iterations
                sleep(1);

                i2cTransaction.slaveAddress = 0x53;
                i2cTransaction.writeBuf = localWritetxBuffer;
                i2cTransaction.writeCount = 0;
                i2cTransaction.readBuf = localWriterxBuffer;
                i2cTransaction.readCount = 1;

        //If this transaction fails, we cannot read from the register
        //This would mean that there was a bus fault, and would give us the error statement seen in the else statement below
                if(I2C_transfer(i2c, &i2cTransaction)){

                    interruptVariable = localWriterxBuffer[0];

                    Display_printf(display, 0, 0, "[%x]: %x \n", 0x30, interruptVariable);
                }
                else
                {
                    Display_printf(display, 0, 0, "I2C Bus fault\n");
                }
                inactivityValue == interruptVariable & 0000 1000;

                if(inactivityValue == 8){
                    Display_printf(display, 0, 0, "Inactivity sensed\n");

                    Display_printf(display, 0, 0, "Current Orientation\n");

                    for(i = 2; i < 8; i++){
                            if(i == 2){ //Reading from register 0x32 -> x0 register (1st 8 bits from the x-axis reading)
                                uint8_t localWritetxBuffer[1];
                                localWritetxBuffer[0] = X_Axis_Register_DATAX0; //0x32
                                uint8_t localWriterxBuffer[1];
                                i2cTransaction.slaveAddress = 0x53;

                                i2cTransaction.writeBuf = localWritetxBuffer;
                                i2cTransaction.writeCount = 1;

                                i2cTransaction.readBuf = localWriterxBuffer;
                                i2cTransaction.readCount = 0;

                                I2C_transfer(i2c, &i2cTransaction);

                                //This sleep(1) is only here for debugging purposes, so that the serial is not slammed all at once with data
                                //can be removed in final iterations
                                sleep(1);

                                i2cTransaction.slaveAddress = 0x53;

                                i2cTransaction.writeBuf = localWritetxBuffer;
                                i2cTransaction.writeCount = 0;

                                i2cTransaction.readBuf = localWriterxBuffer;
                                i2cTransaction.readCount = 1;

                                //If this transaction fails, we cannot read from the register
                                //This would mean that there was a bus fault, and would give us the error statement seen in the else statement below
                                if(I2C_transfer(i2c, &i2cTransaction)){

                                x0 = localWriterxBuffer[0];

                                //Print the value once it is received, this is for debugging purposes to ensure we are getting values from the register

                                Display_printf(display, 0, 0, "[%x]: %x \n", X_Axis_Register_DATAX0, x0);
                                }
                                else
                                {
                                Display_printf(display, 0, 0, "I2C Bus fault\n");
                                }
                                }

                            if(i == 3){ //Reading from register 0x33 -> x1 register (Last 8 bits from the x-axis reading)
                                uint8_t localWritetxBuffer[1];
                                localWritetxBuffer[0] = X_Axis_Register_DATAX1; //0x33
                                uint8_t localWriterxBuffer[1];
                                i2cTransaction.slaveAddress = 0x53;

                                i2cTransaction.writeBuf = localWritetxBuffer;
                                i2cTransaction.writeCount = 1;

                                i2cTransaction.readBuf = localWriterxBuffer;
                                i2cTransaction.readCount = 0;

                                I2C_transfer(i2c, &i2cTransaction);

                                //This sleep(1) is only here for debugging purposes, so that the serial is not slammed all at once with data
                                //can be removed in final iterations
                                sleep(1);

                                i2cTransaction.slaveAddress = 0x53;

                                i2cTransaction.writeBuf = localWritetxBuffer;
                                i2cTransaction.writeCount = 0;

                                i2cTransaction.readBuf = localWriterxBuffer;
                                i2cTransaction.readCount = 1;

                                //If this transaction fails, we cannot read from the register
                                //This would mean that there was a bus fault, and would give us the error statement seen in the else statement below
                                if(I2C_transfer(i2c, &i2cTransaction)){

                                //We want to shift this value 8 bits to the left, as it will be the most significant byte in the 16 bit data received
                                x1 = (localWriterxBuffer[0] << 8);

                                //Print the value once it is received, this is for debugging purposes to ensure we are getting values from the register

                                Display_printf(display, 0, 0, "[%x]: %x \n", X_Axis_Register_DATAX1, x1);
                                }
                                else
                                {
                                Display_printf(display, 0, 0, "I2C Bus fault\n");
                                }
                                }

                            if(i == 4){ //Reading from register 0x34 -> y0 register (1st 8 bits from the y-axis reading)
                                uint8_t localWritetxBuffer[1];
                                localWritetxBuffer[0] = Y_Axis_Register_DATAY0; //0x34
                                uint8_t localWriterxBuffer[1];
                                i2cTransaction.slaveAddress = 0x53;

                                i2cTransaction.writeBuf = localWritetxBuffer;
                                i2cTransaction.writeCount = 1;

                                i2cTransaction.readBuf = localWriterxBuffer;
                                i2cTransaction.readCount = 0;

                                I2C_transfer(i2c, &i2cTransaction);

                                //This sleep(1) is only here for debugging purposes, so that the serial is not slammed all at once with data
                                //can be removed in final iterations
                                sleep(1);

                                i2cTransaction.slaveAddress = 0x53;

                                i2cTransaction.writeBuf = localWritetxBuffer;
                                i2cTransaction.writeCount = 0;

                                i2cTransaction.readBuf = localWriterxBuffer;
                                i2cTransaction.readCount = 1;

                                //If this transaction fails, we cannot read from the register
                                //This would mean that there was a bus fault, and would give us the error statement seen in the else statement below
                                if(I2C_transfer(i2c, &i2cTransaction)){

                                y0 = localWriterxBuffer[0];

                                //Print the value once it is received, this is for debugging purposes to ensure we are getting values from the register

                                Display_printf(display, 0, 0, "[%x]: %x \n", Y_Axis_Register_DATAY0, y0);
                                }
                                else
                                {
                                Display_printf(display, 0, 0, "I2C Bus fault\n");
                                }
                            }

                            if(i == 5){  //Reading from register 0x35 -> y1 register (Last 8 bits from the y-axis reading)
                                uint8_t localWritetxBuffer[1];
                                localWritetxBuffer[0] = Y_Axis_Register_DATAY1; //0x35
                                uint8_t localWriterxBuffer[1];
                                i2cTransaction.slaveAddress = 0x53;
                                                //1 byte is going to be sent when doing a single byte read (Register)
                                i2cTransaction.writeBuf = localWritetxBuffer;
                                i2cTransaction.writeCount = 1;
                                                //1 byte is going to be sent when doing a single byte read (Register)
                                i2cTransaction.readBuf = localWriterxBuffer;
                                i2cTransaction.readCount = 0;

                                I2C_transfer(i2c, &i2cTransaction);

                                //This sleep(1) is only here for debugging purposes, so that the serial is not slammed all at once with data
                                //can be removed in final iterations
                                sleep(1);

                                i2cTransaction.slaveAddress = 0x53;
                                                        //1 byte is going to be sent when doing a single byte read (Register)
                                i2cTransaction.writeBuf = localWritetxBuffer;
                                i2cTransaction.writeCount = 0;
                                                        //1 byte is going to be sent when doing a single byte read (Register)
                                i2cTransaction.readBuf = localWriterxBuffer;
                                i2cTransaction.readCount = 1;

                                //If this transaction fails, we cannot read from the register
                                //This would mean that there was a bus fault, and would give us the error statement seen in the else statement below
                                if(I2C_transfer(i2c, &i2cTransaction)){

                                //We want to shift this value 8 bits to the left, as it will be the most significant byte in the 16 bit data received
                                y1 = (localWriterxBuffer[0] << 8);

                                //Print the value once it is received, this is for debugging purposes to ensure we are getting values from the register
                                Display_printf(display, 0, 0, "[%x]: %x \n", Y_Axis_Register_DATAY1, y1);
                                }
                                else
                                {
                                Display_printf(display, 0, 0, "I2C Bus fault\n");
                                }
                            }

                            if(i == 6){ //Reading from register 0x36 -> z0 register (1st 8 bits from the z-axis reading)
                                uint8_t localWritetxBuffer[1];
                                localWritetxBuffer[0] = Z_Axis_Register_DATAZ0; //0x36
                                uint8_t localWriterxBuffer[1];
                                i2cTransaction.slaveAddress = 0x53;

                                i2cTransaction.writeBuf = localWritetxBuffer;
                                i2cTransaction.writeCount = 1;

                                i2cTransaction.readBuf = localWriterxBuffer;
                                i2cTransaction.readCount = 0;

                                I2C_transfer(i2c, &i2cTransaction);

                                //This sleep(1) is only here for debugging purposes, so that the serial is not slammed all at once with data
                                //can be removed in final iterations
                                sleep(1);

                                i2cTransaction.slaveAddress = 0x53;

                                i2cTransaction.writeBuf = localWritetxBuffer;
                                i2cTransaction.writeCount = 0;

                                i2cTransaction.readBuf = localWriterxBuffer;
                                i2cTransaction.readCount = 1;

                                //If this transaction fails, we cannot read from the register
                                //This would mean that there was a bus fault, and would give us the error statement seen in the else statement below
                                if(I2C_transfer(i2c, &i2cTransaction)){


                                z0 = localWriterxBuffer[0];

                                //Print the value once it is received, this is for debugging purposes to ensure we are getting values from the register
                                Display_printf(display, 0, 0, "[%x]: %x \n", Z_Axis_Register_DATAZ0, z0);
                                }
                                else
                                {
                                Display_printf(display, 0, 0, "I2C Bus fault\n");
                                }
                            }

                            if(i == 7){ //Reading from register 0x37 -> z1 register (Last 8 bits from the z-axis reading)
                                uint8_t localWritetxBuffer[1];
                                localWritetxBuffer[0] = Z_Axis_Register_DATAZ1; //0x37
                                uint8_t localWriterxBuffer[1];
                                i2cTransaction.slaveAddress = 0x53;

                                i2cTransaction.writeBuf = localWritetxBuffer;
                                i2cTransaction.writeCount = 1;

                                i2cTransaction.readBuf = localWriterxBuffer;
                                i2cTransaction.readCount = 0;

                                I2C_transfer(i2c, &i2cTransaction);

                                //This sleep(1) is only here for debugging purposes, so that the serial is not slammed all at once with data
                                //can be removed in final iterations
                                sleep(1);

                                i2cTransaction.slaveAddress = 0x53;

                                i2cTransaction.writeBuf = localWritetxBuffer;
                                i2cTransaction.writeCount = 0;

                                i2cTransaction.readBuf = localWriterxBuffer;
                                i2cTransaction.readCount = 1;

                                //If this transaction fails, we cannot read from the register
                                //This would mean that there was a bus fault, and would give us the error statement seen in the else statement below
                                if(I2C_transfer(i2c, &i2cTransaction)){

                                //We want to shift this value 8 bits to the left, as it will be the most significant byte in the 16 bit data received
                                z1 = (localWriterxBuffer[0] << 8);

                                //Print the value once it is received, this is for debugging purposes to ensure we are getting values from the register
                                Display_printf(display, 0, 0, "[%x]: %x \n", Z_Axis_Register_DATAZ1, z1);
                                }
                                else
                                {
                                Display_printf(display, 0, 0, "I2C Bus fault\n");
                                }
                            }

                        //This sleep(1) is only here for debugging purposes, so that the serial is not slammed all at once with data
                        //can be removed in final iterations
                        sleep(1);

                        } //end of for loop

                        //Since the x1 was masked to the left 8 bits already, we can just add the two bytes together to get the 16 bit data we need
                        //Then multiply it by the 4mg/LSB as specified by the ADXL345 data sheet, which is determined by the range we used
                        xf=(x0)+(x1);
                        newx_out = xf*0.004;


                        //Since the y1 was masked to the left 8 bits already, we can just add the two bytes together to get the 16 bit data we need
                        //Then multiply it by the 4mg/LSB as specified by the ADXL345 data sheet, which is determined by the range we used
                        yf=(y0)+(y1);
                        newy_out = yf*0.004;


                        //Since the z1 was masked to the left 8 bits already, we can just add the two bytes together to get the 16 bit data we need
                        //Then multiply it by the 4mg/LSB as specified by the ADXL345 data sheet, which is determined by the range we used
                        zf=(z0)+(z1);
                        newz_out = zf*0.004;

                        //Now print these values to the serial. Used for debugging purposes, can be removed in final iterations
                        Display_printf(display, 0, 0, "x [%d]: %f g\n", xf, newx_out);
                        Display_printf(display, 0, 0, "y [%d]: %f g\n", yf, newy_out);
                        Display_printf(display, 0, 0, "z [%d]: %f g\n", zf, newz_out);

                        if((x_out!= newx_out) && (y_out!= newy_out) && (z_out!= newz_out)){

                            Display_printf(display, 0, 0, "Orientation different than before\n");
                            Display_printf(display, 0, 0, "User has Fallen!!");

                        }
                }
            }
        }


 /*
 *      OR if we want to be sure user has fallen, we can take another reading of the accelerometer Axes register and compare
 *          them to the old values we just got before the fall.
 *
 *      Run new for loop, store the values in new variables, and see if they are different than before, if they are
 *      This means that the person most likely, has fallen and we again can write:
 */


}

/*
 *          Note: This separate reading of the accelerometer axes would again ONLY happen, if the other three if statements have
 *          been fulfilled.
 *
 */

} //end of while loop

sleep(2);

} //end of initial i2c transaction if statement setting the ADXL345 in measurement mode,
    //if it did not go through we run the following else statement signifying a bus fault
else
{
Display_printf(display, 0, 0, "I2C Bus fault\n");
}
    /* Sleep for 1 second */
sleep(1);

    /* De-initialized I2C */

I2C_close(i2c);

Display_printf(display, 0, 0, "I2C closed!\n");

return (NULL);

}


/*
 *  ========================== Free-fall Evaluations ========================
 *
 *  Start of the fall: The phenomenon of weightlessness will always occur at the start of a fall.
 *  It will become more significant during free fall, and the vector sum of acceleration will tend toward 0 g;
 *  the duration of that condition will depend on the height of freefall. Even though weightlessness during an ordinary fall
 *  is not as significant as that during a freefall, the vector sum of acceleration will still be substantially less than 1 g
 *  (while it is generally greater than 1 g under normal conditions). Therefore, this is the first basis for determining the fall
 *  status that could be detected by the ADXL345’s FREE_FALL interrupt.

    Impact: After experiencing weightlessness, the human body will impact the ground or other objects;
    the acceleration curve shows this as a large shock. This shock is detected by the ACTIVITY interrupt of ADXL345.
    Therefore, the second basis for determining a fall is the ACTIVITY interrupt right after the FREE_FALL interrupt.

    Aftermath: Generally speaking, the human body, after falling and making impact, can not rise immediately; rather it
    remains in a motionless position for a short period (or longer as a possible sign of unconsciousness).
    On the acceleration curve, this presents as an interval of flat line, and is detected by the INACTIVITY interrupt of ADXL345.
    Therefore, the third basis for determining a fall situation is the INACTIVITY interrupt after the ACTIVITY interrupt.

    Comparing before and after: After a fall, the individual’s body will be in a different orientation than before,
    so the static acceleration in three axes will be different from the initial status before the fall.
    Suppose that the fall detector is belt-wired on the individual’s body, to provide the entire history of acceleration,
    including the initial status. We can read the acceleration data in all three axes after the INACTIVITY interrupt and
    compare those sampling data with the initial status.

    The combination of these qualifications forms the entire fall-detection algorithm, which, when exercised,
    can cause the system to raise an appropriate alert that a fall has occurred. Of course, the time interval between
    interrupts has to be within a reasonable range. Normally, the time interval between FREE_FALL interrupt (weightlessness) and
    ACTIVITY interrupt (impact) is not very long unless one is falling from the top of a very high building!
    Similarly, the time interval between ACTIVITY interrupt (impact) and INACTIVITY interrupt (essentially motionless)
    should not be very long. A practical example will be given in the next section with a set of reasonable values.
    The related interrupt detection threshold and time parameters can be flexibly set as needed.

    If a fall results in serious consequences, such as unconsciousness,
    the human body will remain motionless for an even longer period of time,
    a status that can still be detected by the INACTIVITY interrupt, so a second critical alert could be sent out
    if the inactive state was detected to continue for a defined long period of time after a fall.

    Using this knowledge, we should initialize the following registers to the following values:

    0x24 -> 0x08 -> establishing it to have a 0.5g activity threshold (could also set it to 0x20 which is a 2g threshold)
    0x25 -> 0x03 -> set inactivity threshold to be 0.1875g
    0x26 -> 0x02 -> set inactivity time to be 2 seconds (could be also 0x0A for 10 seconds if we want more time)
    0x27 -> 0x7F/0xFF -> Enable Activity and Inactivity of X-, Y-, Z-axis, wherein Inactivity is ac-coupled mode,
                                Activity is dc-coupled/ac-coupled mode
    0x28 -> 0x0C -> Set free-fall threshold as 0.75g
    0x29 -> 0x06 -> Set free-fall threshold time as 30ms
    0x2C -> 0x0A -> Keep as default ODR = 100Hz
    0x2E -> 0x1C -> Enable Activity, Inactivity, Free-Fall interrupts
    0x31 -> 0x0B -> Set as ±16 g measurement range, 13-bit right alignment, high level interrupt trigger, I2C interface

    This is all done in the same way we established the PCR (0x2D) and wrote to the register to put the ADXL in
        measurement mode. All of these registers follow the same initialization protocols:







 */


//potential initialization of all the registers within the ADXL345

/**
void adxl345_init(int fd){

    i2cTransaction.slaveAddress = (0x53); //address of the ADXL345 specified in data sheet
        //two bytes are going to be sent when doing a single byte write (Register + Value)
    i2cTransaction.writeBuf = txBuffer;
    i2cTransaction.writeCount = 2;
        //No reply from the slave when doing a write, no need for a buffer
    i2cTransaction.readBuf = NULL;
    i2cTransaction.readCount = 0;

    I2C_transfer(i2c, &i2cTransaction))


        //full resolution mode, where the output resolution increases with the g range
         // set by the range bits to maintain a 4 mg/LSB scale factor.
          //Also sets ADXL into +-16g range
    wiringPiI2CWriteReg8(fd, 0x31, 0x0b);

    wiringPiI2CWriteReg8(fd, 0x2d, 0x08); //Puts ADXL into measurement mode

    wiringPiI2CWriteReg8(fd, 0x2e, 0x00); //keeps all respective functions from generating interrupts
    wiringPiI2CWriteReg8(fd, 0x1e, 0x00); //x-axis offset
    wiringPiI2CWriteReg8(fd, 0x1f, 0x00); //y-axis offset
    wiringPiI2CWriteReg8(fd, 0x20, 0x00); //z-axis offset
    wiringPiI2CWriteReg8(fd, 0x21, 0x00); //value of 0 disables the single/double tap function
    wiringPiI2CWriteReg8(fd, 0x22, 0x00); //value of 0 disables the double tap function
    wiringPiI2CWriteReg8(fd, 0x23, 0x00); //value of 0 disables the double tap function

        //Thresh_ACT holds the threshold for how much the accelerometer must move to be considered active
         //The scale factor is 62.5 mg/LSB so a value of 1 = 0.0625g means that the ADXL must be moving more than 0.0625g
          //to be active
    wiringPiI2CWriteReg8(fd, 0x24, 0x01);

        //Thresh_INACT holds the threshold for detecting inactivity.
         //the magnitude of the inactivity event is compared with the value in the register.
          // The scale factor is 62.5 mg/LSB so a value of 15 = 0.9375g.
    wiringPiI2CWriteReg8(fd, 0x25, 0x0f);

        //Time_INACT holds the threshold for time the ADXL must be inactive before inactivity is declared.
         // A value of 43 means the accelerometer must be inactive 43 seconds before inactivity is declared
    wiringPiI2CWriteReg8(fd, 0x26, 0x2b);

    wiringPiI2CWriteReg8(fd, 0x27, 0x00); //ensures all axes do not participate in active/inactive detection

        //Thresh_FF. The scale factor is 62.5 mg/LSB so a value of 9 = 0.5625g. Compares all axes to this number to determine
         //free-fall
    wiringPiI2CWriteReg8(fd, 0x28, 0x09);

        //Time_FF, represents the minimum time that the value pf all the axes must be less than Thresh_FF to get a
         // free-fall interrupt. Scale factor is 5ms/LSB, thus a value of 255 = 1.275 seconds.
    wiringPiI2CWriteReg8(fd, 0x29, 0xff);

        //A value of 0x80 does nothing in this register, may as well be set to 0x00 since bit[7] for this register
         //isn't set to mean anything
    wiringPiI2CWriteReg8(fd, 0x2a, 0x80);

        //Sets the device bandwidth and output data rate (see table 7 and 8 in ADXL345 data-sheet for more info on setting
         //these values). A value of 0x0A = 1010 is the default value which translates to an ODR of 100Hz
    wiringPiI2CWriteReg8(fd, 0x2c, 0x0a);

    wiringPiI2CWriteReg8(fd, 0x2f, 0x00); //sends all interrupts to the INT1 pin

        //Sets the FIFO into Stream Mode. This means that the FIFO holds the last 32 data values. When FIFO is full,
         // the oldest data is replaced with newer data. A value of 11111 in the "samples" bits means that 31 FIFO
          // entries are needed to trigger a watermark interrupt
    wiringPiI2CWriteReg8(fd, 0x38, 0x9f);

}

*/
