/*
 * pulseTask.c
 *
 *  Created on: Mar 7, 2021
 *      Author: AveryW
 */



/*********************************************************************
 * INCLUDES
 */
#include <string.h>

#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Clock.h>
#include <ti/sysbios/knl/Event.h>
#include <ti/sysbios/knl/Queue.h>
#include <ti/sysbios/knl/Semaphore.h>

#include <unistd.h>
#include <stdint.h>
#include <stddef.h>
#include <stdbool.h>
#include <stdio.h>

/* XDC module Headers */
#include <xdc/std.h>
#include <xdc/runtime/System.h>
#include <xdc/cfg/global.h> //needed to link a bunch of stuff together
#include <xdc/runtime/Error.h>


#include <ti/display/Display.h>

#if !(defined __TI_COMPILER_VERSION__)
#include <intrinsics.h>
#endif

#include <ti/drivers/utils/List.h>

#include <icall.h>
#include "util.h"
#include <bcomdef.h>
/* This Header file contains all BLE API and icall structure definition */
#include <icall_ble_api.h>

#include <devinfoservice.h>
#include <simple_gatt_profile.h>

#ifdef USE_RCOSC
#include <rcosc_calibration.h>
#endif //USE_RCOSC

#include <ti_drivers_config.h>
#include <board_key.h>

#include <menu/two_btn_menu.h>

#include "simple_peripheral_menu.h"
#include "simple_peripheral.h"
#include "ti_ble_config.h"

#ifdef PTM_MODE
#include "npi_task.h"               // To allow RX event registration
#include "npi_ble.h"                // To enable transmission of messages to UART
#include "icall_hci_tl.h"   // To allow ICall HCI Transport Layer
#endif // PTM_MODE


#include "ti_drivers_config.h"
#include <ti/drivers/Board.h>
#include <ti/drivers/GPIO.h>
#include <ti/drivers/I2C.h>
#include <ti/display/Display.h>

#include "pulseService.h"
#include "pulseTask.h"
#include "bio_sensor.h"


/*********************************************************************
 * MACROS
 */



/*********************************************************************
 * CONSTANTS
 */
// Task configuration
#define PT_TASK_PRIORITY                     1

#ifndef PT_TASK_STACK_SIZE
#define PT_TASK_STACK_SIZE                   1536
#endif

// Internal Events for RTOS application
#define PT_ICALL_EVT                         ICALL_MSG_EVENT_ID // Event_Id_31
#define PT_QUEUE_EVT                         UTIL_QUEUE_EVENT_ID // Event_Id_30

// Bitwise OR of all RTOS events to pend on
#define PT_ALL_EVENTS                        (PT_ICALL_EVT             | \
                                              PT_QUEUE_EVT)


#define PULSE_SAMPLE_ARRAY_SIZE 10 //number of pulseSample samples per array

/*********************************************************************
 * TYPEDEFS
 */

// App event passed from stack modules. This type is defined by the application
// since it can queue events to itself however it wants.
typedef struct
{
  uint8_t event;                // event type
  void    *pData;               // pointer to message
} etEvt_t;


struct pulseSample{
    uint32_t sampleTime;
    struct bioData pulseData;
};

/*********************************************************************
 * GLOBAL VARIABLES
 */
// Task configuration
Task_Struct ptTask;
#if defined __TI_COMPILER_VERSION__
#pragma DATA_ALIGN(ptTaskStack, 8)
#else
#pragma data_alignment=8
#endif
uint8_t ptTaskStack[PT_TASK_STACK_SIZE];



extern I2C_Handle i2c; //external I2C handle object
extern I2C_Params i2cParams; //external I2C parameters object
extern I2C_Transaction gi2cTransaction; //external I2C transaction object

/*********************************************************************
 * LOCAL VARIABLES
 */

// Entity ID globally used to check for source and/or destination of messages
static ICall_EntityID selfEntity;

// Event globally used to post local events and pend on system and
// local events.
static ICall_SyncHandle syncEvent;

// Queue object used for app messages
static Queue_Struct appMsgQueue;

struct pulseSample pulseArray[PULSE_SAMPLE_ARRAY_SIZE];
uint8_t pulseIndex = 0;

struct pulseSample singlePulseSample;


/*********************************************************************
 * LOCAL FUNCTIONS
 */

static void pulseTask_init(void);
static void pulseTask_taskFxn(UArg a0, UArg a1);

/*********************************************************************
 * EXTERN FUNCTIONS
 */
extern void AssertHandler(uint8 assertCause, uint8 assertSubcause);

/*********************************************************************
 * PROFILE CALLBACKS
 */



/*********************************************************************
 * @fn      SimplePeripheral_createTask
 *
 * @brief   Task creation function for the Simple Peripheral.
 */
void pulseTask_createTask(void)
{
  Task_Params taskParams;

  // Configure task
  Task_Params_init(&taskParams);
  taskParams.stack = ptTaskStack;
  taskParams.stackSize = PT_TASK_STACK_SIZE;
  taskParams.priority = PT_TASK_PRIORITY;

  Task_construct(&ptTask, pulseTask_taskFxn, &taskParams, NULL);
}

/*********************************************************************
 * @fn      pulseTask_init
 *
 * @brief   Called during initialization and contains application
 *          specific initialization (ie. hardware initialization/setup,
 *          table initialization, power up notification, etc), and
 *          profile initialization/setup.
 */
static void pulseTask_init(void)
{


  // ******************************************************************
  // N0 STACK API CALLS CAN OCCUR BEFORE THIS CALL TO ICall_registerApp
  // ******************************************************************
  // Register the current thread as an ICall dispatcher application
  // so that the application can send and receive messages.
  ICall_registerApp(&selfEntity, &syncEvent);

  PulseService_AddService(selfEntity);
  uint8_t pulseService_pulseSamples_initVal[PULSESERVICE_PULSESAMPLES_LEN] = {0};
  PulseService_SetParameter(PULSESERVICE_PULSESAMPLES_ID, PULSESERVICE_PULSESAMPLES_LEN, pulseService_pulseSamples_initVal);

  GPIO_init();

}

/*********************************************************************
 * @fn      pulseTask_taskFxn
 *
 * @brief   Application task entry point for the an external test
 *
 * @param   a0, a1 - not used.
 */
static void pulseTask_taskFxn(UArg a0, UArg a1)
{
  // Initialize application
    pulseTask_init();

    singlePulseSample.pulseData.reserveOne = 0xFF; //set the terminator to be something obvious

    uint8_t readStatus = 0; //status byte
    uint8_t deviceMode = 0;
    uint8_t status = 0;

    GPIO_setConfig(Board_GPIO_LED_0, GPIO_CFG_OUT_STD | GPIO_CFG_OUT_LOW);
    GPIO_setConfig(Board_GPIO_LED_1, GPIO_CFG_OUT_STD | GPIO_CFG_OUT_LOW);

    GPIO_setConfig(Board_GPIO_DIO0_MFIO, GPIO_CFG_OUT_STD | GPIO_CFG_OUT_HIGH); //mfio pin, pull high
    GPIO_setConfig(Board_GPIO_DIO1_RESET, GPIO_CFG_OUT_STD | GPIO_CFG_OUT_HIGH); //reset pin, pull low

    //hardware reset the MAX32664
    GPIO_write(Board_GPIO_DIO1_RESET, 0x0); //pull reset low
    usleep(10000); //sleep for 10 ms
    GPIO_write(Board_GPIO_DIO1_RESET, 0x1); //set the reset pin high
    sleep(1); //sleep for 1 second
    GPIO_setConfig(Board_GPIO_DIO0_MFIO, GPIO_CFG_IN_PU); //setup the MFIO as an input so MAX32664 can use it

    deviceMode = beginI2C(i2c, &readStatus);
    if(readStatus != SUCCESS){ //if we had issues beginning the I2C communication with the pulse oximeter
        GPIO_write(Board_GPIO_LED_0, CONFIG_GPIO_LED_ON); //turn the red LED on
    }
    else{
        GPIO_write(Board_GPIO_LED_0, CONFIG_GPIO_LED_OFF); //turn the red LED off
    }

    readStatus = configMAX32664(SENSOR_AND_ALGORITHM, MODE_TWO, 1); //configure the MAX32664 to output sensor and algorithm data in algorithm mode 2
    if(readStatus != SUCCESS){ //while we had issues configuring the MAX32664, try again
//        readStatus = configMAX32664(SENSOR_AND_ALGORITHM, MODE_TWO, 1); //configure the MAX32664 to output sensor and algorithm data in algorithm mode 2
        GPIO_write(Board_GPIO_LED_0, CONFIG_GPIO_LED_ON); //turn the red LED on
    }


    while(1){
        Semaphore_pend(semPulse, BIOS_WAIT_FOREVER); //pend on the read sensor semaphore that is released by the clock

        singlePulseSample.sampleTime = Clock_getTicks();
        singlePulseSample.pulseData = readSensorData(&status); //take a pulse oximeter reading
        singlePulseSample.pulseData.reserveOne = 0xFF; //set the terminator to be something obvious

        pulseArray[pulseIndex] = singlePulseSample;
        pulseIndex += 1;

        if(status != SUCCESS){ //if we had a pulse oximeter transaction error
    //        softwareResetMAX30101(); //software reset the MAX30101
            softwareResetMAX32664(); //software reset the MAX32664
            status = configMAX32664(SENSOR_AND_ALGORITHM, MODE_TWO, 1); //configure the MAX32664 to output sensor and algorithm data in algorithm mode 2
            if(status != SUCCESS){ //while we had issues configuring the MAX32664, try again
//                status = configMAX32664(SENSOR_AND_ALGORITHM, MODE_TWO, 1); //configure the MAX32664 to output sensor and algorithm data in algorithm mode 2
                GPIO_write(Board_GPIO_LED_0, CONFIG_GPIO_LED_ON); //turn the red LED on
            }
        }

        if(gi2cTransaction.status != SUCCESS && status != I2C_STATUS_QUEUED){ //if we had an I2C bus error and it's not because it's queued
            I2C_close(i2c); //close the I2C bus
//            usleep(1000); //sleep for 1ms
            i2c = I2C_open(CONFIG_I2C_0, &i2cParams); //open the on-board I2C channel
            if (i2c == NULL) {
//                while (1); //wait forever
                GPIO_write(Board_GPIO_LED_0, CONFIG_GPIO_LED_ON); //turn the red LED on
            }
            softwareResetMAX32664(); //software reset the MAX32664
            deviceMode = beginI2C(i2c, &status);
            if(status != SUCCESS){ //if we had issues beginning the I2C communication with the pulse oximeter
                GPIO_write(Board_GPIO_LED_0, CONFIG_GPIO_LED_ON); //turn the red LED on
            }
            else{
                GPIO_write(Board_GPIO_LED_0, CONFIG_GPIO_LED_OFF); //turn the red LED off
            }

            status = configMAX32664(SENSOR_AND_ALGORITHM, MODE_TWO, 1); //configure the MAX32664 to output sensor and algorithm data in algorithm mode 2
            if(status != SUCCESS){ //while we had issues configuring the MAX32664, try again
//                status = configMAX32664(SENSOR_AND_ALGORITHM, MODE_TWO, 1); //configure the MAX32664 to output sensor and algorithm data in algorithm mode 2
                GPIO_write(Board_GPIO_LED_0, CONFIG_GPIO_LED_ON); //turn the red LED on
            }
        }


        if(pulseIndex >= PULSE_SAMPLE_ARRAY_SIZE){
            //set the adc characteristic value (past #ADC_SAMPLE_ARRAY_SIZE samples)
            PulseService_SetParameter(PULSESERVICE_PULSESAMPLES_ID,
                                      PULSESERVICE_PULSESAMPLES_LEN,
                                      pulseArray);
            pulseIndex = 0;
        }


    }

}

void pulseClockSwiFxn(UArg arg0){
    Semaphore_post(semPulse);
}




