/**********************************************************************************************
 * Filename:       sensorService.c
 *
 * Description:    This file contains the implementation of the service.
 *
 * Copyright (c) 2015-2019, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *************************************************************************************************/


/*********************************************************************
 * INCLUDES
 */
#include <string.h>

#include <icall.h>

/* This Header file contains all BLE API and icall structure definition */
#include "icall_ble_api.h"

#include "sensorService.h"

/*********************************************************************
 * MACROS
 */

/*********************************************************************
 * CONSTANTS
 */

/*********************************************************************
 * TYPEDEFS
 */

/*********************************************************************
* GLOBAL VARIABLES
*/

// sensorService Service UUID
CONST uint8_t sensorServiceUUID[ATT_BT_UUID_SIZE] =
{
  LO_UINT16(SENSORSERVICE_SERV_UUID), HI_UINT16(SENSORSERVICE_SERV_UUID)
};

// accelSamples UUID
CONST uint8_t sensorService_AccelSamplesUUID[ATT_UUID_SIZE] =
{
  TI_BASE_UUID_128(SENSORSERVICE_ACCELSAMPLES_UUID)
};
// edaSample UUID
CONST uint8_t sensorService_EdaSampleUUID[ATT_UUID_SIZE] =
{
  TI_BASE_UUID_128(SENSORSERVICE_EDASAMPLE_UUID)
};
// adcSamples UUID
CONST uint8_t sensorService_AdcSamplesUUID[ATT_UUID_SIZE] =
{
  TI_BASE_UUID_128(SENSORSERVICE_ADCSAMPLES_UUID)
};

/*********************************************************************
 * LOCAL VARIABLES
 */

static sensorServiceCBs_t *pAppCBs = NULL;

/*********************************************************************
* Profile Attributes - variables
*/

// Service declaration
static CONST gattAttrType_t sensorServiceDecl = { ATT_BT_UUID_SIZE, sensorServiceUUID };

// Characteristic "AccelSamples" Properties (for declaration)
static uint8_t sensorService_AccelSamplesProps = GATT_PROP_READ | GATT_PROP_NOTIFY;

// Characteristic "AccelSamples" Value variable
static uint8_t sensorService_AccelSamplesVal[SENSORSERVICE_ACCELSAMPLES_LEN] = {0};

// Characteristic "AccelSamples" CCCD
static gattCharCfg_t *sensorService_AccelSamplesConfig;
// Characteristic "EdaSample" Properties (for declaration)
static uint8_t sensorService_EdaSampleProps = GATT_PROP_READ | GATT_PROP_NOTIFY;

// Characteristic "EdaSample" Value variable
static uint8_t sensorService_EdaSampleVal[SENSORSERVICE_EDASAMPLE_LEN] = {0};

// Characteristic "EdaSample" CCCD
static gattCharCfg_t *sensorService_EdaSampleConfig;
// Characteristic "AdcSamples" Properties (for declaration)
static uint8_t sensorService_AdcSamplesProps = GATT_PROP_READ | GATT_PROP_NOTIFY;

// Characteristic "AdcSamples" Value variable
static uint8_t sensorService_AdcSamplesVal[SENSORSERVICE_ADCSAMPLES_LEN] = {0};

// Characteristic "AdcSamples" CCCD
static gattCharCfg_t *sensorService_AdcSamplesConfig;

/*********************************************************************
* Profile Attributes - Table
*/

static gattAttribute_t sensorServiceAttrTbl[] =
{
  // sensorService Service Declaration
  {
    { ATT_BT_UUID_SIZE, primaryServiceUUID },
    GATT_PERMIT_READ,
    0,
    (uint8_t *)&sensorServiceDecl
  },
    // AccelSamples Characteristic Declaration
    {
      { ATT_BT_UUID_SIZE, characterUUID },
      GATT_PERMIT_READ,
      0,
      &sensorService_AccelSamplesProps
    },
      // AccelSamples Characteristic Value
      {
        { ATT_UUID_SIZE, sensorService_AccelSamplesUUID },
        GATT_PERMIT_READ,
        0,
        sensorService_AccelSamplesVal
      },
      // AccelSamples CCCD
      {
        { ATT_BT_UUID_SIZE, clientCharCfgUUID },
        GATT_PERMIT_READ | GATT_PERMIT_WRITE,
        0,
        (uint8 *)&sensorService_AccelSamplesConfig
      },
    // EdaSample Characteristic Declaration
    {
      { ATT_BT_UUID_SIZE, characterUUID },
      GATT_PERMIT_READ,
      0,
      &sensorService_EdaSampleProps
    },
      // EdaSample Characteristic Value
      {
        { ATT_UUID_SIZE, sensorService_EdaSampleUUID },
        GATT_PERMIT_READ,
        0,
        sensorService_EdaSampleVal
      },
      // EdaSample CCCD
      {
        { ATT_BT_UUID_SIZE, clientCharCfgUUID },
        GATT_PERMIT_READ | GATT_PERMIT_WRITE,
        0,
        (uint8 *)&sensorService_EdaSampleConfig
      },
    // AdcSamples Characteristic Declaration
    {
      { ATT_BT_UUID_SIZE, characterUUID },
      GATT_PERMIT_READ,
      0,
      &sensorService_AdcSamplesProps
    },
      // AdcSamples Characteristic Value
      {
        { ATT_UUID_SIZE, sensorService_AdcSamplesUUID },
        GATT_PERMIT_READ,
        0,
        sensorService_AdcSamplesVal
      },
      // AdcSamples CCCD
      {
        { ATT_BT_UUID_SIZE, clientCharCfgUUID },
        GATT_PERMIT_READ | GATT_PERMIT_WRITE,
        0,
        (uint8 *)&sensorService_AdcSamplesConfig
      },
};

/*********************************************************************
 * LOCAL FUNCTIONS
 */
static bStatus_t sensorService_ReadAttrCB( uint16_t connHandle, gattAttribute_t *pAttr,
                                           uint8_t *pValue, uint16_t *pLen, uint16_t offset,
                                           uint16_t maxLen, uint8_t method );
static bStatus_t sensorService_WriteAttrCB( uint16_t connHandle, gattAttribute_t *pAttr,
                                            uint8_t *pValue, uint16_t len, uint16_t offset,
                                            uint8_t method );

/*********************************************************************
 * PROFILE CALLBACKS
 */
// Simple Profile Service Callbacks
CONST gattServiceCBs_t sensorServiceCBs =
{
  sensorService_ReadAttrCB,  // Read callback function pointer
  sensorService_WriteAttrCB, // Write callback function pointer
  NULL                       // Authorization callback function pointer
};

/*********************************************************************
* PUBLIC FUNCTIONS
*/

/*
 * SensorService_AddService- Initializes the SensorService service by registering
 *          GATT attributes with the GATT server.
 *
 */
extern bStatus_t SensorService_AddService( uint8_t rspTaskId )
{
  uint8_t status;

  // Allocate Client Characteristic Configuration table
  sensorService_AccelSamplesConfig = (gattCharCfg_t *)ICall_malloc( sizeof(gattCharCfg_t) * linkDBNumConns );
  if ( sensorService_AccelSamplesConfig == NULL )
  {
    return ( bleMemAllocError );
  }

  // Initialize Client Characteristic Configuration attributes
  GATTServApp_InitCharCfg( LINKDB_CONNHANDLE_INVALID, sensorService_AccelSamplesConfig );
  // Allocate Client Characteristic Configuration table
  sensorService_EdaSampleConfig = (gattCharCfg_t *)ICall_malloc( sizeof(gattCharCfg_t) * linkDBNumConns );
  if ( sensorService_EdaSampleConfig == NULL )
  {
    return ( bleMemAllocError );
  }

  // Initialize Client Characteristic Configuration attributes
  GATTServApp_InitCharCfg( LINKDB_CONNHANDLE_INVALID, sensorService_EdaSampleConfig );
  // Allocate Client Characteristic Configuration table
  sensorService_AdcSamplesConfig = (gattCharCfg_t *)ICall_malloc( sizeof(gattCharCfg_t) * linkDBNumConns );
  if ( sensorService_AdcSamplesConfig == NULL )
  {
    return ( bleMemAllocError );
  }

  // Initialize Client Characteristic Configuration attributes
  GATTServApp_InitCharCfg( LINKDB_CONNHANDLE_INVALID, sensorService_AdcSamplesConfig );
  // Register GATT attribute list and CBs with GATT Server App
  status = GATTServApp_RegisterService( sensorServiceAttrTbl,
                                        GATT_NUM_ATTRS( sensorServiceAttrTbl ),
                                        GATT_MAX_ENCRYPT_KEY_SIZE,
                                        &sensorServiceCBs );

  return ( status );
}

/*
 * SensorService_RegisterAppCBs - Registers the application callback function.
 *                    Only call this function once.
 *
 *    appCallbacks - pointer to application callbacks.
 */
bStatus_t SensorService_RegisterAppCBs( sensorServiceCBs_t *appCallbacks )
{
  if ( appCallbacks )
  {
    pAppCBs = appCallbacks;

    return ( SUCCESS );
  }
  else
  {
    return ( bleAlreadyInRequestedMode );
  }
}

/*
 * SensorService_SetParameter - Set a SensorService parameter.
 *
 *    param - Profile parameter ID
 *    len - length of data to right
 *    value - pointer to data to write.  This is dependent on
 *          the parameter ID and WILL be cast to the appropriate
 *          data type (example: data type of uint16 will be cast to
 *          uint16 pointer).
 */
bStatus_t SensorService_SetParameter( uint8_t param, uint16_t len, void *value )
{
  bStatus_t ret = SUCCESS;
  switch ( param )
  {
    case SENSORSERVICE_ACCELSAMPLES_ID:
      if ( len == SENSORSERVICE_ACCELSAMPLES_LEN )
      {
        memcpy(sensorService_AccelSamplesVal, value, len);

        // Try to send notification.
        GATTServApp_ProcessCharCfg( sensorService_AccelSamplesConfig, (uint8_t *)&sensorService_AccelSamplesVal, FALSE,
                                    sensorServiceAttrTbl, GATT_NUM_ATTRS( sensorServiceAttrTbl ),
                                    INVALID_TASK_ID,  sensorService_ReadAttrCB);
      }
      else
      {
        ret = bleInvalidRange;
      }
      break;

    case SENSORSERVICE_EDASAMPLE_ID:
      if ( len == SENSORSERVICE_EDASAMPLE_LEN )
      {
        memcpy(sensorService_EdaSampleVal, value, len);

        // Try to send notification.
        GATTServApp_ProcessCharCfg( sensorService_EdaSampleConfig, (uint8_t *)&sensorService_EdaSampleVal, FALSE,
                                    sensorServiceAttrTbl, GATT_NUM_ATTRS( sensorServiceAttrTbl ),
                                    INVALID_TASK_ID,  sensorService_ReadAttrCB);
      }
      else
      {
        ret = bleInvalidRange;
      }
      break;

    case SENSORSERVICE_ADCSAMPLES_ID:
      if ( len == SENSORSERVICE_ADCSAMPLES_LEN )
      {
        memcpy(sensorService_AdcSamplesVal, value, len);

        // Try to send notification.
        GATTServApp_ProcessCharCfg( sensorService_AdcSamplesConfig, (uint8_t *)&sensorService_AdcSamplesVal, FALSE,
                                    sensorServiceAttrTbl, GATT_NUM_ATTRS( sensorServiceAttrTbl ),
                                    INVALID_TASK_ID,  sensorService_ReadAttrCB);
      }
      else
      {
        ret = bleInvalidRange;
      }
      break;

    default:
      ret = INVALIDPARAMETER;
      break;
  }
  return ret;
}


/*
 * SensorService_GetParameter - Get a SensorService parameter.
 *
 *    param - Profile parameter ID
 *    value - pointer to data to write.  This is dependent on
 *          the parameter ID and WILL be cast to the appropriate
 *          data type (example: data type of uint16 will be cast to
 *          uint16 pointer).
 */
bStatus_t SensorService_GetParameter( uint8_t param, uint16_t *len, void *value )
{
  bStatus_t ret = SUCCESS;
  switch ( param )
  {
    default:
      ret = INVALIDPARAMETER;
      break;
  }
  return ret;
}


/*********************************************************************
 * @fn          sensorService_ReadAttrCB
 *
 * @brief       Read an attribute.
 *
 * @param       connHandle - connection message was received on
 * @param       pAttr - pointer to attribute
 * @param       pValue - pointer to data to be read
 * @param       pLen - length of data to be read
 * @param       offset - offset of the first octet to be read
 * @param       maxLen - maximum length of data to be read
 * @param       method - type of read message
 *
 * @return      SUCCESS, blePending or Failure
 */
static bStatus_t sensorService_ReadAttrCB( uint16_t connHandle, gattAttribute_t *pAttr,
                                       uint8_t *pValue, uint16_t *pLen, uint16_t offset,
                                       uint16_t maxLen, uint8_t method )
{
  bStatus_t status = SUCCESS;

  // See if request is regarding the AccelSamples Characteristic Value
if ( ! memcmp(pAttr->type.uuid, sensorService_AccelSamplesUUID, pAttr->type.len) )
  {
    if ( offset > SENSORSERVICE_ACCELSAMPLES_LEN )  // Prevent malicious ATT ReadBlob offsets.
    {
      status = ATT_ERR_INVALID_OFFSET;
    }
    else
    {
      *pLen = MIN(maxLen, SENSORSERVICE_ACCELSAMPLES_LEN - offset);  // Transmit as much as possible
      memcpy(pValue, pAttr->pValue + offset, *pLen);
    }
  }
  // See if request is regarding the EdaSample Characteristic Value
else if ( ! memcmp(pAttr->type.uuid, sensorService_EdaSampleUUID, pAttr->type.len) )
  {
    if ( offset > SENSORSERVICE_EDASAMPLE_LEN )  // Prevent malicious ATT ReadBlob offsets.
    {
      status = ATT_ERR_INVALID_OFFSET;
    }
    else
    {
      *pLen = MIN(maxLen, SENSORSERVICE_EDASAMPLE_LEN - offset);  // Transmit as much as possible
      memcpy(pValue, pAttr->pValue + offset, *pLen);
    }
  }
  // See if request is regarding the AdcSamples Characteristic Value
else if ( ! memcmp(pAttr->type.uuid, sensorService_AdcSamplesUUID, pAttr->type.len) )
  {
    if ( offset > SENSORSERVICE_ADCSAMPLES_LEN )  // Prevent malicious ATT ReadBlob offsets.
    {
      status = ATT_ERR_INVALID_OFFSET;
    }
    else
    {
      *pLen = MIN(maxLen, SENSORSERVICE_ADCSAMPLES_LEN - offset);  // Transmit as much as possible
      memcpy(pValue, pAttr->pValue + offset, *pLen);
    }
  }
  else
  {
    // If we get here, that means you've forgotten to add an if clause for a
    // characteristic value attribute in the attribute table that has READ permissions.
    *pLen = 0;
    status = ATT_ERR_ATTR_NOT_FOUND;
  }

  return status;
}


/*********************************************************************
 * @fn      sensorService_WriteAttrCB
 *
 * @brief   Validate attribute data prior to a write operation
 *
 * @param   connHandle - connection message was received on
 * @param   pAttr - pointer to attribute
 * @param   pValue - pointer to data to be written
 * @param   len - length of data
 * @param   offset - offset of the first octet to be written
 * @param   method - type of write message
 *
 * @return  SUCCESS, blePending or Failure
 */
static bStatus_t sensorService_WriteAttrCB( uint16_t connHandle, gattAttribute_t *pAttr,
                                        uint8_t *pValue, uint16_t len, uint16_t offset,
                                        uint8_t method )
{
  bStatus_t status  = SUCCESS;
  uint8_t   paramID = 0xFF;

  // See if request is regarding a Client Characterisic Configuration
  if ( ! memcmp(pAttr->type.uuid, clientCharCfgUUID, pAttr->type.len) )
  {
    // Allow only notifications.
    status = GATTServApp_ProcessCCCWriteReq( connHandle, pAttr, pValue, len,
                                             offset, GATT_CLIENT_CFG_NOTIFY);
  }
  else
  {
    // If we get here, that means you've forgotten to add an if clause for a
    // characteristic value attribute in the attribute table that has WRITE permissions.
    status = ATT_ERR_ATTR_NOT_FOUND;
  }

  // Let the application know something changed (if it did) by using the
  // callback it registered earlier (if it did).
  if (paramID != 0xFF)
    if ( pAppCBs && pAppCBs->pfnChangeCb )
      pAppCBs->pfnChangeCb(connHandle, paramID, len, pValue); // Call app function from stack task context.

  return status;
}
