/*
 * Copyright (c) 2015-2019, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 *  ======== clock.c ========
 */

#include <unistd.h>
#include <stdint.h>
#include <stddef.h>
#include <stdbool.h>
#include <stdio.h>

/* XDC module Headers */
#include <xdc/std.h>
#include <xdc/runtime/System.h>
#include <xdc/cfg/global.h> //needed to link a bunch of stuff together


/* BIOS module Headers */
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Clock.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Semaphore.h>
#include <ti/sysbios/gates/GateTask.h>


/* Driver configuration */
#include "ti_drivers_config.h"
#include <ti/drivers/Board.h>
#include <ti/drivers/GPIO.h>
#include <ti/drivers/I2C.h>
#include <ti/drivers/SPI.h>
#include <ti/drivers/ADC.h>
#include <ti/display/Display.h>

#include "bio_sensor.h"

#define X_Axis_Register_DATAX0 0x32 /*X-Axis Data 0 */
#define X_Axis_Register_DATAX1 0x33 /*X-Axis Data 1 */
#define Y_Axis_Register_DATAY0 0x34 /*Y-Axis Data 0 */
#define Y_Axis_Register_DATAY1 0x35 /*Y-Axis Data 1 */
#define Z_Axis_Register_DATAZ0 0x36 /*Z-Axis Data 0 */
#define Z_Axis_Register_DATAZ1 0x37 /*Z-Axis Data 1 */
#define DEVID 0x00 /* Device address */
#define PCR 0x2D /* power controlled register*/

#define ADC_SAMPLE_COUNT  (1024)

static Display_Handle display;

I2C_Handle      i2c;
I2C_Params      i2cParams;
I2C_Transaction i2cTrans;

SPI_Handle      spi;
SPI_Params      spiParams;
SPI_Transaction spiTransaction;


struct bioData gBody; //global bioData struct
uint8_t gStatusByte = 0; //global status byte

int16_t gAccelX = 0;
int16_t gAccelY = 0;
int16_t gAccelZ = 0;

uint16_t gEDA = 0;
uint16_t gEdaStatus = 0;

struct accelData{
    uint16_t accelX;
    uint16_t accelY;
    uint16_t accelZ;
};


/* ADC conversion result variables */
uint16_t adcValue0[ADC_SAMPLE_COUNT];
uint32_t adcValue0MicroVolt[ADC_SAMPLE_COUNT];
uint16_t adcValue1[ADC_SAMPLE_COUNT];
uint32_t adcValue1MicroVolt[ADC_SAMPLE_COUNT];
uint16_t     i;


uint8_t configAccel(void);
uint8_t readAccelDevID(uint8_t *statusByte);
int16_t readAccelX(uint8_t *statusByte);
int16_t readAccelY(uint8_t *statusByte);
int16_t readAccelZ(uint8_t *statusByte);
uint8_t readAccelReg(uint8_t regAddress, uint8_t *statusByte);
uint16_t readEDA(uint16_t *statusInt);
void ecgConversion(void);
void micConversion(void);
void convertBothChannels(void);

static void spiErrorHandler(SPI_Transaction *transaction,
    Display_Handle display);

/*
 *  ======== main ========
 */
int main()
{
    /* Construct BIOS Objects */
//    Clock_Params clkParams;

    /* Call driver init functions */
    Board_init();
    Display_init();
    GPIO_init();
    I2C_init();
    SPI_init();

    GPIO_setConfig(Board_GPIO_LED_0, GPIO_CFG_OUT_STD | GPIO_CFG_OUT_LOW);
    GPIO_setConfig(Board_GPIO_LED_1, GPIO_CFG_OUT_STD | GPIO_CFG_OUT_LOW);

    GPIO_setConfig(Board_GPIO_DIO0_MFIO, GPIO_CFG_OUT_STD | GPIO_CFG_OUT_HIGH); //mfio pin, pull high
    GPIO_setConfig(Board_GPIO_DIO1_RESET, GPIO_CFG_OUT_STD | GPIO_CFG_OUT_HIGH); //reset pin, pull low

#ifdef CONFIG_GPIO_SPI_CS //if the SPI CS is defined (using software CS)
    GPIO_setConfig(CONFIG_GPIO_SPI_CS, GPIO_CFG_OUT_STD | GPIO_CFG_OUT_HIGH);
#endif


    /* Open the HOST display for output */
    display = Display_open(Display_Type_UART, NULL);
    if (display == NULL) {
        while (1);
    }


    I2C_Params_init(&i2cParams);
    i2cParams.bitRate = I2C_400kHz;
//    i2cParams.bitRate = I2C_100kHz;

    i2c = I2C_open(CONFIG_I2C_0, &i2cParams); //open the on-board I2C channel
    if (i2c == NULL) {
        while (1); //wait forever
    }


    SPI_Params_init(&spiParams);
    spiParams.dataSize = 16;
    //spiParams.bitRate = 400000;
    //spiParams.frameFormat = SPI_POL0_PHA0;
    //spiParams.mode = SPI_MASTER;

    spi = SPI_open(CONFIG_SPI_MASTER, &spiParams); //open the SPI hardware
    if (spi == NULL){
        while(1); //wait forever
    }


    ADC_init();

    BIOS_start();    /* does not return */
    return(0);
}

void task0ReadAccelData(UArg arg0, UArg arg1){
    //task to read the accelerometer data

    UInt32 timeAccel;
    UInt32 timePrintBefore;
    UInt32 timePrintAfter;

    uint8_t accelStatus = configAccel();
    if(accelStatus != SUCCESS){ //if there was an issue configuring the accelerometer
        Display_printf(display, 0, 0, "configAccel failed");
        Display_printf(display, 0, 0, "Accel status: %u ", accelStatus);
    }
    else{
        Display_printf(display, 0, 0, "\n\nconfigAccel passed");
    }

    uint8_t devID = readAccelDevID(&accelStatus);
    if(accelStatus != SUCCESS){ //if there was an issue
        Display_printf(display, 0, 0, "readAccelDevID failed");
        Display_printf(display, 0, 0, "Accel status: %u ", accelStatus);
    }
    else{
        Display_printf(display, 0, 0, "readAccelDevID passed");
        Display_printf(display, 0, 0, "DEVID: 0x%02x ", devID);
    }

    while(1){
        Semaphore_pend(semReadAccel, BIOS_WAIT_FOREVER);


//        Semaphore_pend(semDataProtect, BIOS_WAIT_FOREVER); //pend on the semaphore that's protecting the global data

        Semaphore_pend(semAccelData, BIOS_WAIT_FOREVER); //pend on the semaphore that's protecting the global semaphore data

        //gate object to protect against shared data issues
//        IArg gateKey = GateTask_enter(gateTask0);

//        GPIO_write(Board_GPIO_LED_0, 0x1); //pull LED high
//        usleep(100); //sleep for 100 us
//        GPIO_write(Board_GPIO_LED_0, 0x0); //pull LED low

//        timeAccel = Clock_getTicks();
//        System_printf("System time before accel read = %lu\n", (ULong)timeAccel);
//        System_flush();
        GPIO_toggle(Board_GPIO_LED_0);
        gAccelX =  readAccelX(&accelStatus);
        gAccelY =  readAccelY(&accelStatus);
        gAccelZ =  readAccelZ(&accelStatus);
        GPIO_toggle(Board_GPIO_LED_0);

//        GPIO_write(Board_GPIO_LED_0, 1);
//        GPIO_write(Board_GPIO_LED_1, 1);
//        GPIO_write(Board_GPIO_LED_0, 0);
//        GPIO_write(Board_GPIO_LED_1, 0);
//        GPIO_write(Board_GPIO_LED_0, 1);
//        GPIO_write(Board_GPIO_LED_1, 1);
//        timePrintBefore = Clock_getTicks();
//        Display_printf(display, 0, 0, "$ACCL,%d,%d,%d", gAccelX, gAccelY, gAccelZ);
//        System_printf("$ACCL,%d,%d,%d", gAccelX, gAccelY, gAccelZ);
//        System_flush();
        //        timePrintAfter = Clock_getTicks();
//        GPIO_write(Board_GPIO_LED_0, 0);
//        GPIO_write(Board_GPIO_LED_1, 0);
//        GPIO_write(Board_GPIO_LED_0, 1);
//        GPIO_write(Board_GPIO_LED_1, 1);
//        GPIO_write(Board_GPIO_LED_0, 0);
//        GPIO_write(Board_GPIO_LED_1, 0);
//        GPIO_write(Board_GPIO_LED_0, 0);

//        Display_printf(display, 0, 0, "Time difference: %d, %d", timePrintBefore, timePrintAfter);




//        accelX = readAccelX(&accelStatus);
//        if(accelStatus != SUCCESS){ //if there was an issue
//            Display_printf(display, 0, 0, "readAccelX failed");
//            Display_printf(display, 0, 0, "Accel status: %u ", accelStatus);
//        }
//        else{
//            xOut = (float)accelX * 0.004f;
//            Display_printf(display, 0, 0, "readAccelX passed");
//            Display_printf(display, 0, 0, "accelX: %d [0x%04x] ", accelX, accelX);
//            Display_printf(display, 0, 0, "Xf: %f g ", xOut);
//        }

//        timeAccel = Clock_getTicks();
//        System_printf("System time after accel read = %lu\n", (ULong)timeAccel);
//        System_flush();

//        GPIO_toggle(Board_GPIO_LED_0);

//        GateTask_leave(gateTask0, gateKey); //leave the gate protected area


        Semaphore_post(semAccelData); //post the accel semaphore

//        Semaphore_post(semDataProtect); //release the data protection semaphore

//        Semaphore_post(semDisplayData); //post that there's new bio data to be displayed

    }

}

void task1ReadPulseData(UArg arg0, UArg arg1){
    //task to read the pulse oximeter data
    uint8_t readStatus = 0;
    uint8_t deviceMode = 0;
    UInt32 timePulse;

    //reset the MAX32664
    GPIO_write(Board_GPIO_DIO1_RESET, 0x0); //pull reset low
    usleep(10000); //sleep for 10 ms
    GPIO_write(Board_GPIO_DIO1_RESET, 0x1); //set the reset pin high
    sleep(1); //sleep for 1 second
    GPIO_setConfig(Board_GPIO_DIO0_MFIO, GPIO_CFG_IN_PU); //setup the MFIO as an input so MAX32664 can use it

    deviceMode = beginI2C(i2c, &readStatus);
    if(readStatus != SUCCESS){ //if we had issues beginning the I2C communication with the pulse oximeter
        GPIO_write(Board_GPIO_LED_1, CONFIG_GPIO_LED_ON); //turn the red LED on
//        while(1){
//            //enter an infinite broken loop
//            //will cause the RTOS to hang, so comment out if needed
//        }
    }

    readStatus = configMAX32664(SENSOR_AND_ALGORITHM, MODE_TWO, 1); //configure the MAX32664 to output sensor and algorithm data in algorithm mode 2
//    while(readStatus != SUCCESS){ //while we had issues configuring the MAX32664, try again
//        readStatus = configMAX32664(SENSOR_AND_ALGORITHM, MODE_TWO, 1); //configure the MAX32664 to output sensor and algorithm data in algorithm mode 2
//    }

    while(1){
        Semaphore_pend(semReadPulse, BIOS_WAIT_FOREVER); //pend on the read sensor semaphore that is released by the clock

//        timePulse = Clock_getTicks();
//        System_printf("System time in task1ReadPulseData = %lu\n", (ULong)timePulse);
//        System_flush();

        Semaphore_pend(semDataProtect, BIOS_WAIT_FOREVER); //pend on the semaphore that's protecting the global data

        //perform a sensor reading
        gBody = readSensorData(&readStatus); //read the sensor data
        gStatusByte = readStatus; //copy over the status byte

        Semaphore_post(semDataProtect); //release the data protection semaphore

        if(readStatus != SUCCESS){
            //debug point
            readStatus = 5;
        }


//        Semaphore_post(semDisplayData); //post that there's new bio data to be displayed
    }
}


void task2ReadEdaData(UArg arg0, UArg arg1){

    uint16_t spiTranStatus = 0;

    UInt32 timeEDA;


    while(1){
        Semaphore_pend(semReadEDA, BIOS_WAIT_FOREVER); //pend on the read sensor semaphore that is released by the clock

//        timeEDA = Clock_getTicks();
//        System_printf("System time in task2ReadEdaData = %lu\n", (ULong)timeEDA);
//        System_flush();

        Semaphore_pend(semEdaData, BIOS_WAIT_FOREVER); //pend on the semaphore protecting the global EDA data

        gEDA = readEDA(&gEdaStatus);

        Semaphore_post(semEdaData); //post the eda semaphore

//        Semaphore_post(semDisplayData); //post that there's new data to be displayed
    }

}


void task3DisplaySensorData(UArg agr0, UArg arg1){

    struct bioData localBody; //local copy of the bio data
    uint8_t localStatusByte = 0; //local status byte
    int16_t localX = 0; //local accelerometer X
    int16_t localY = 0;
    int16_t localZ = 0;
    float localXfloat = 0.0;
    float localYfloat = 0.0;
    float localZfloat = 0.0;
    uint16_t localEdaInt = 0;
    uint16_t localEdaStatus = 0;
    float localEdaFloat = 0.0;
    UInt32 time;

    while(1){
        Semaphore_pend(semDisplayData, BIOS_WAIT_FOREVER); //pend on the display data semaphore

        Semaphore_pend(semDataProtect, BIOS_WAIT_FOREVER); //pend on the semaphore that protects the bio data
        //body data
        localBody = gBody; //create a local copy of the global bio data
        localStatusByte = gStatusByte;
        Semaphore_post(semDataProtect); //post the data protection semaphore, since we made a local copy

        Semaphore_pend(semAccelData, BIOS_WAIT_FOREVER); //pend on the semaphore that's protecting the global accel data
        localX = gAccelX;
        localY = gAccelY;
        localZ = gAccelZ;
        Semaphore_post(semAccelData); //post the accel semaphore

        Semaphore_pend(semEdaData, BIOS_WAIT_FOREVER); //pend on the semaphore that's protecting the global EDA data
        localEdaInt = gEDA;
        localEdaStatus = gEdaStatus;
        Semaphore_post(semEdaData); //post the EDA semaphore


        localXfloat = (float)localX * 0.004f;
        localYfloat = (float)localY * 0.004f;
        localZfloat = (float)localZ * 0.004f;

        localEdaInt = localEdaInt >> 1;
        localEdaInt &= 0x0fff; //just want the first 12 bits, so mask out the top 4
        localEdaFloat = (float)localEdaInt * 3.3f * 0.000244f; //convert to a voltage by multiplying Int by Vcc (3.3V) and then dividing by 2^12 (ADC resolution)

        Display_printf(display, 0, 0, "\n\nNew Data: "); //new sensor printing

        if(localStatusByte != SUCCESS){ //if the I2C had transaction issues
            Display_printf(display, 0, 0, "readSensorData Failed."); //print that there was an issue with the function
            Display_printf(display, 0, 0, "Error: 0x%02x", localStatusByte);
        }
        else{
            Display_printf(display, 0, 0, "IR LED Count: %u ", localBody.irLed); //print out the IR LED count
            Display_printf(display, 0, 0, "Red LED Count: %u ", localBody.redLed); //print out the Red LED count
            Display_printf(display, 0, 0, "Heart Rate: %02u ", localBody.heartRate); //print out the heart rate
            Display_printf(display, 0, 0, "HR Confidence: %u ", localBody.confidence); //print our HR confidence level
            Display_printf(display, 0, 0, "SpO2 Level: %02u ", localBody.oxygen); //print out SpO2 level
            Display_printf(display, 0, 0, "Algorithm state: %u ", localBody.status); //print algorithm state
            Display_printf(display, 0, 0, "Algorithm status: %d ", localBody.extStatus); //print algorithm state
            Display_printf(display, 0, 0, "Blood Oxygen R value: %.2f ", localBody.rValue); //print algorithm state
        }

        Display_printf(display, 0, 0, "accelX: %d [0x%04x] ", localX, localX);
        Display_printf(display, 0, 0, "Xf: %f g ", localXfloat);
        Display_printf(display, 0, 0, "accelY: %d [0x%04x] ", localY, localY);
        Display_printf(display, 0, 0, "Yf: %f g ", localYfloat);
        Display_printf(display, 0, 0, "accelZ: %d [0x%04x] ", localZ, localZ);
        Display_printf(display, 0, 0, "Zf: %f g ", localZfloat);

        if(localEdaStatus != SUCCESS){
            Display_printf(display, 0, 0, "SPI transaction error"); //print that there was an issue with the SPI transaction
            spiErrorHandler(&spiTransaction, display);
        }
        Display_printf(display, 0, 0, "EDA Voltage: %f V ", localEdaFloat);

        System_flush();
    }
}

void task4ReadADCData(UArg arg0, UArg arg1){

    while(1){
        Semaphore_pend(semReadADC, BIOS_WAIT_FOREVER);
        GPIO_toggle(Board_GPIO_LED_1);
        convertBothChannels();
        GPIO_toggle(Board_GPIO_LED_1);

    }
}



void clockReadPulse(UArg arg0){ //clock that posts the semaphore to read the pulse oximeter
//    UInt32 time;
//
//    time = Clock_getTicks();
//    System_printf("System time in clockReadPulse = %lu\n", (ULong)time);
//    System_flush();

//    GPIO_toggle(Board_GPIO_LED_0);


    Semaphore_post(semReadPulse);
}

void clockReadAccel(UArg arg0){ //clock that posts the semaphore to read the accelerometer (~100Hz)
//    UInt32 time;
//
//    time = Clock_getTicks();
//    System_printf("System time in clockReadAccel = %lu\n", (ULong)time);
//    System_flush();

    Semaphore_post(semReadAccel);
}

void clockReadEDA(UArg arg0){
//    UInt32 time;
//
//    time = Clock_getTicks();
//    System_printf("System time in clockReadEDA = %lu\n", (ULong)time);
//    System_flush();

//    GPIO_toggle(Board_GPIO_LED_0);

    Semaphore_post(semReadEDA);
}


void clockReadADC(UArg arg0){
    Semaphore_post(semReadADC);
}

uint8_t configAccel(void){
    //configure the accelerometer for use
    uint8_t tempTxBuffer[2];

    //populate the txBuffer before doing the I2C_transaction
    tempTxBuffer[0] = PCR;
    //set bit 4 and 3 [0000 1000] for measurement mode enable
    tempTxBuffer[1] = 0x08;

    i2cTrans.slaveAddress = (0x53); //address of the ADXL345 specified in data sheet
    //two bytes are going to be sent when doing a single byte write (Register + Value)
    i2cTrans.writeBuf = tempTxBuffer;
    i2cTrans.writeCount = 2;
    //No reply from the slave when doing a write, no need for a buffer
    i2cTrans.readBuf = NULL;
    i2cTrans.readCount = 0;

    if(I2C_transfer(i2c, &i2cTrans)){ //if there was a successful I2C transaction
        return SUCCESS; //return the status
    }
    else{ //else we had an error
        return ERR_UNKNOWN; //return an error
    }
}


uint8_t readAccelDevID(uint8_t *statusByte){
    //reads the accelerometer device ID (DEVID). Should be 0xE5

    uint8_t devID = readAccelReg(DEVID, statusByte);
    if(*statusByte != SUCCESS){
        devID = 0;
    }

    return devID;

}


int16_t readAccelX(uint8_t *statusByte){
    //read the X axis data

    uint8_t x0 = 0; //LSB of X data
    uint8_t x1 = 0; //MSB of X data
    int16_t accelX;

    x0 = readAccelReg(X_Axis_Register_DATAX0, statusByte);
    if(*statusByte != SUCCESS){
        return 0;
    }

    x1 = readAccelReg(X_Axis_Register_DATAX1, statusByte);
    if(*statusByte != SUCCESS){
        return 0;
    }

    accelX = (int16_t)x1 << 8; //shift over the MSB of data
    accelX |= (int16_t)x0; //bitwise OR the LSB

    return accelX;
}


int16_t readAccelY(uint8_t *statusByte){
    //read the Y axis data

    uint8_t y0 = 0; //LSB of Y data
    uint8_t y1 = 0; //MSB of Y data
    int16_t accelY;

    y0 = readAccelReg(Y_Axis_Register_DATAY0, statusByte);
    if(*statusByte != SUCCESS){
        return 0;
    }

    y1 = readAccelReg(Y_Axis_Register_DATAY1, statusByte);
    if(*statusByte != SUCCESS){
        return 0;
    }

    accelY = (int16_t)y1 << 8; //shift over the MSB of data
    accelY |= (int16_t)y0; //bitwise OR the LSB

    return accelY;
}


int16_t readAccelZ(uint8_t *statusByte){
    //read the Z axis data

    uint8_t z0 = 0; //LSB of Z data
    uint8_t z1 = 0; //MSB of Z data
    int16_t accelZ;

    z0 = readAccelReg(Z_Axis_Register_DATAZ0, statusByte);
    if(*statusByte != SUCCESS){
        return 0;
    }

    z1 = readAccelReg(Z_Axis_Register_DATAZ1, statusByte);
    if(*statusByte != SUCCESS){
        return 0;
    }

    accelZ = (int16_t)z1 << 8; //shift over the MSB of data
    accelZ |= (int16_t)z0; //bitwise OR the LSB

    return accelZ;
}


uint8_t readAccelReg(uint8_t regAddress, uint8_t *statusByte){
    //reads a single byte from a single register

    uint8_t localTxBuffer = regAddress; //set the Register Address
    uint8_t localRxBuffer = 0; //Read byte at register
    i2cTrans.slaveAddress = (0x53);
    i2cTrans.writeBuf = &localTxBuffer;
    i2cTrans.writeCount = 1;
    i2cTrans.readBuf = &localRxBuffer;
    i2cTrans.readCount = 1;

    if(I2C_transfer(i2c, &i2cTrans)){ //if there was a successful transaction
        *statusByte = SUCCESS;
        return localRxBuffer;
    }
    else{ //else there was an error
        *statusByte = i2cTrans.status;
        return 0;
    }
}


uint16_t readEDA(uint16_t *statusInt){

    uint16_t txBuffer;
    uint16_t readVal;
    uint8_t transactionOk;

    spiTransaction.txBuf = &txBuffer;
    spiTransaction.rxBuf = &readVal;
    spiTransaction.count = 1;

#ifdef CONFIG_GPIO_SPI_CS //if the SPI CS is defined (using software CS)
    GPIO_write(CONFIG_GPIO_SPI_CS, 0); //pull the CS low
#endif
    transactionOk = SPI_transfer(spi, &spiTransaction);
#ifdef CONFIG_GPIO_SPI_CS //if the SPI CS is defined (using software CS)
    GPIO_write(CONFIG_GPIO_SPI_CS, 1); //pull CS high
#endif
    *statusInt = spiTransaction.status;

    if(!transactionOk){ //if there was an error with the SPI transaction
        return 0; //return 0
    }
    else{ //else no issue
        return readVal; //return the read value
    }

}

void convertBothChannels(void){
//    GPIO_toggle(Board_GPIO_LED_0);
    //better version of Karim's thing
    ecgConversion();
    micConversion();
    i++;
    if(i >= ADC_SAMPLE_COUNT){
        i = 0;
    }
//    GPIO_toggle(Board_GPIO_LED_0);
}

void ecgConversion(void)
{
    ADC_Handle   adc;
    ADC_Params   params;
    int_fast16_t res;


    ADC_Params_init(&params);
    adc = ADC_open(CONFIG_ADC_ECG, &params);

    if (adc == NULL) {
        Display_printf(display, 0, 0, "Error initializing CONFIG_ADC_0\n");
        while (1);
    }
    /* Blocking mode conversion */
    res = ADC_convert(adc, &adcValue0[i]);

    if(res == ADC_STATUS_SUCCESS) {
        adcValue0MicroVolt[i] = ADC_convertRawToMicroVolts(adc, adcValue0[i]);
    }

    ADC_close(adc);
}


void micConversion(void){

    ADC_Handle   adc;
    ADC_Params   params;
    int_fast16_t res;

    ADC_Params_init(&params);
    adc = ADC_open(CONFIG_ADC_MIC, &params);

    if(adc == NULL) {
        Display_printf(display, 0, 0, "Error initializing CONFIG_ADC_1\n");
        while (1);
    }

    res = ADC_convert(adc, &adcValue1[i]);

    if(res == ADC_STATUS_SUCCESS) {
        adcValue1MicroVolt[i] = ADC_convertRawToMicroVolts(adc, adcValue1[i]);
    }

    ADC_close(adc);
}


/*
 *  ======== spiErrorHandler ========
 */
static void spiErrorHandler(SPI_Transaction *transaction,
    Display_Handle display)
{
    switch (transaction->status) {
    case SPI_TRANSFER_COMPLETED:
        Display_printf(display, 0, 0, "SPI transfer completed!");
        break;
    case SPI_TRANSFER_STARTED:
        Display_printf(display, 0, 0, "SPI transfer started and in progress!");
        break;
    case SPI_TRANSFER_CANCELED:
        Display_printf(display, 0, 0, "SPI transfer was canceled");
        break;
    case SPI_TRANSFER_FAILED:
        Display_printf(display, 0, 0, "SPI transfer failed!");
        break;
    case SPI_TRANSFER_CSN_DEASSERT:
        Display_printf(display, 0, 0, "SPI chip select was de-asserted!");
        break;
    case SPI_TRANSFER_PEND_CSN_ASSERT:
        Display_printf(display, 0, 0, "SPI transfer is pending until chip select is asserted!");
        break;
    case SPI_TRANSFER_QUEUED:
        Display_printf(display, 0, 0, "SPI transfer added to transaction queue!");
        break;
    default:
        Display_printf(display, 0, 0, "SPI undefined error case!");
        break;
    }
}


